@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -396,17 +396,7 @@ cache_read()
 	filename = build_filename();
 	if (!os_exists(filename))
 		return;
-	fp = fopen(filename, "rb");
-	if (!fp)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("open $filename: $errno"));
-		/* NOTREACHED */
-	}
+	fp = fopen_and_check(filename, "rb");
 
 	/*
 	 * Take a shared lock: many processes may read simultaneously.
@@ -417,16 +407,7 @@ cache_read()
 	 * get the number of entries in the file
 	 */
 	if (fread_sane(fp, &nitems, sizeof(nitems)))
-	{
-		sub_context_ty	*scp;
-
-		bomb:
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("read $filename: $errno"));
-		/* NOTREACHED */
-	}
+		fatal_intl_read(filename);
 
 	/*
 	 * read each entry in the file
@@ -434,7 +415,7 @@ cache_read()
 	for (j = 0; j < nitems; ++j)
 	{
 		if (cache_read_item(fp))
-			goto bomb;
+			fatal_intl_read(filename);
 	}
 
 	/*
@@ -445,16 +426,7 @@ cache_read()
 	/*
 	 * all done
 	 */
-	if (fclose(fp))
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("close $filename: $errno"));
-		/* NOTREACHED */
-	}
+	fclose_and_check(fp, filename);
 }
 
 
@@ -622,17 +594,7 @@ cache_write()
 	 * open the cache file
 	 */
 	filename = build_filename();
-	fp = fopen(filename, "wb");
-	if (!fp)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("open $filename: $errno"));
-		/* NOTREACHED */
-	}
+	fp = fopen_and_check(filename, "wb");
 
 	/*
 	 * Take an exclusive lock: only one process may take the lock at
@@ -646,23 +608,13 @@ cache_write()
 	 * write the number of entries to the file
 	 */
 	if (fwrite_sane(fp, &symtab->hash_load, sizeof(symtab->hash_load)))
-		goto bomb;
+		fatal_intl_write(filename);
 
 	/*
 	 * write each cache entry to the file
 	 */
 	symtab_walk(symtab, walk, fp);
-	if (ferror(fp))
-	{
-		sub_context_ty	*scp;
-
-		bomb:
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("write $filename: $errno"));
-		/* NOTREACHED */
-	}
+	fflush_and_check(fp, filename);
 
 	/*
 	 * Release the file lock.
@@ -672,16 +624,7 @@ cache_write()
 	/*
 	 * close the cache file
 	 */
-	if (fclose(fp))
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%s", filename);
-		fatal_intl(scp, i18n("close $filename: $errno"));
-		/* NOTREACHED */
-	}
+	fclose_and_check(fp, filename);
 }
 
 
