@@ -23,6 +23,7 @@
 #include <ac/ctype.h>
 #include <ac/string.h>
 
+#include <input.h>
 #include <lang_c.h>
 #include <mem.h>
 #include <str_list.h>
@@ -142,7 +143,7 @@ directive(s, type1, type2)
  *	lang_c_scan
  *
  * SYNOPSIS
- *	int lang_c_scan(FILE *fp, string_list_ty *type1, string_list_ty *type2);
+ *	int lang_c_scan(input_ty *fp, string_list_ty *type1, string_list_ty *type2);
  *
  * DESCRIPTION
  *	The lang_c_scan function is used to scan a file looking
@@ -160,11 +161,11 @@ directive(s, type1, type2)
  *		-1 on file errors
  */
 
-static int lang_c_scan _((FILE *, string_list_ty *, string_list_ty *));
+static int lang_c_scan _((input_ty *, string_list_ty *, string_list_ty *));
 
 static int
 lang_c_scan(fp, type1, type2)
-	FILE		*fp;
+	input_ty		*fp;
 	string_list_ty	*type1;
 	string_list_ty	*type2;
 {
@@ -188,15 +189,10 @@ type2 = %08lX)\n{\n"/*}*/, fp, type1, type2));
 			max += 80;
 			line = mem_change_size(line, max);
 		}
-		c = getc(fp);
+		c = input_getc(fp);
 		switch (c)
 		{
-		case EOF:
-			if (ferror(fp))
-			{
-				result = -1;
-				break;
-			}
+		case INPUT_EOF:
 			if (!pos)
 				break;
 			/* fall through... */
