@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  */
 
 #include <ac/stddef.h>
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/string.h>
 #include <ac/stdlib.h>
 
@@ -62,7 +62,9 @@ enum
 	arglex_token_absent_system_error,
 	arglex_token_absent_system_ignore,
 	arglex_token_absent_system_mention,
+	arglex_token_absent_use_this,
 	arglex_token_escape_newline,
+	arglex_token_exclude,
 	arglex_token_include,
 	arglex_token_language,
 	arglex_token_lang_c,
@@ -124,6 +126,26 @@ static	arglex_table_ty	argtab[] =
 		"-Absent_System_Mention",
 		(arglex_token_ty)arglex_token_absent_system_mention,
 	},
+	{
+		"-Absent_Use_This",
+		(arglex_token_ty)arglex_token_absent_use_this,
+	},
+	{
+		"-Generated_Files",
+		(arglex_token_ty)arglex_token_absent_use_this,
+	},
+	{
+		"-Interior_Files",
+		(arglex_token_ty)arglex_token_absent_use_this,
+	},
+	{
+		"-Derived_Files",
+		(arglex_token_ty)arglex_token_absent_use_this,
+	},
+	{
+		"-\\U*",
+		(arglex_token_ty)arglex_token_absent_use_this,
+	},
 	{
 		"-C",
 		(arglex_token_ty)arglex_token_lang_c,
@@ -136,6 +158,10 @@ static	arglex_table_ty	argtab[] =
 		"-Escape_Newlines",
 		(arglex_token_ty)arglex_token_escape_newline,
 	},
+	{
+		"-EXclude",
+		(arglex_token_ty)arglex_token_exclude,
+	},
 	{
 		"-\\I*",
 		(arglex_token_ty)arglex_token_include,
@@ -223,7 +249,6 @@ main(argc, argv)
 	sniff_ty	*language;
 	char		*arg1;
 	char		*arg2;
-	sub_context_ty	*scp;
 
 	arglex_init(argc, argv, argtab);
 	str_initialize();
@@ -276,21 +301,8 @@ main(argc, argv)
 			if (option.o_absent_local != -1)
 			{
 				duplicate:
-				scp = sub_context_new();
-				sub_var_set
-				(
-					scp,
-					"Name",
-					"%s",
-					arglex_value.alv_string
-				);
-				error_intl
-				(
-					scp,
-					i18n("duplicate \"$name\" option")
-				);
-				sub_context_delete(scp);
-				usage();
+				arg_duplicate_cur(usage);
+				/* NOTREACHED */
 			}
 			option.o_absent_local = absent_ignore;
 			break;
@@ -337,21 +349,32 @@ main(argc, argv)
 			option.o_absent_program = absent_error;
 			break;
 
+		case arglex_token_absent_use_this:
+			for (;;)
+			{
+				switch (arglex())
+				{
+				default:
+					break;
+	
+				case arglex_token_string:
+					sniff_use_this(arglex_value.alv_string);
+					continue;
+	
+				case arglex_token_stdio:
+					sniff_use_this_cut();
+					continue;
+				}
+				break;
+			}
+			continue;
+
 		case arglex_token_include:
 			switch (arglex())
 			{
 			default:
-				scp = sub_context_new();
-				sub_var_set
-				(
-					scp,
-					"Name",
-					"%s",
-					arglex_token_name(arglex_token_include)
-				);
-				error_intl(scp, i18n("$name needs string"));
-				sub_context_delete(scp);
-				usage();
+				arg_needs_string(arglex_token_include, usage);
+				/* NOTREACHED */
 
 			case arglex_token_string:
 				sniff_include(arglex_value.alv_string);
@@ -392,17 +415,8 @@ main(argc, argv)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
 			{
-				scp = sub_context_new();
-				sub_var_set
-				(
-					scp,
-					"Name",
-					"%s",
-					arglex_token_name(arglex_token_language)
-				);
-				error_intl(scp, i18n("$name needs string"));
-				sub_context_delete(scp);
-				usage();
+				arg_needs_string(arglex_token_language, usage);
+				/* NOTREACHED */
 			}
 			language = lang_from_name(arglex_value.alv_string);
 			break;
@@ -421,13 +435,13 @@ main(argc, argv)
 		
 		case arglex_token_prefix:
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_prefix, usage);
 			sniff_prefix_set(arglex_value.alv_string);
 			break;
 		
 		case arglex_token_suffix:
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_suffix, usage);
 			sniff_suffix_set(arglex_value.alv_string);
 			break;
 
@@ -435,22 +449,22 @@ main(argc, argv)
 			if (option.output)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_output, usage);
 			option.output = arglex_value.alv_string;
 			break;
 
 		case arglex_token_remove_leading_path:
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_remove_leading_path, usage);
 			sniff_remove_leading_path(arglex_value.alv_string);
 			break;
 
 		case arglex_token_substitute_leading_path:
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_substitute_leading_path, usage);
 			arg1 = arglex_value.alv_string;
 			if (arglex() != arglex_token_string)
-				usage();
+				arg_needs_string(arglex_token_substitute_leading_path, usage);
 			arg2 = arglex_value.alv_string;
 			sniff_substitute_leading_path(arg1, arg2);
 			break;
@@ -484,6 +498,13 @@ main(argc, argv)
 				goto duplicate;
 			option.escape_newline = 1;
 			break;
+
+		case arglex_token_exclude:
+			arglex();
+			if (arglex_token != arglex_token_string)
+				arg_needs_string(arglex_token_exclude, usage);
+			sniff_exclude(arglex_value.alv_string);
+			break;
 		}
 		arglex();
 	}
