@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,13 @@
  */
 
 #include <ac/stddef.h>
-#include <stdio.h>
 #include <ac/string.h>
-#include <errno.h>
+#include <ac/errno.h>
 #include <signal.h>
 
 #include <cache.h>
 #include <error_intl.h>
+#include <input/file_text.h>
 #include <mem.h>
 #include <os.h>
 #include <sniff.h>
@@ -47,8 +47,10 @@ static	char		*suffix;
 static	long		pcount;
 static	string_list_ty	srl1;
 static	string_list_ty	srl2;
+static	string_list_ty	use_these;
 static	string_list_ty	visited;
 static	string_list_ty	remove_path;
+static	string_list_ty	exclude;
 static	sniff_ty	*lang;
 static	int		has_been_cut;
 static	int		interrupted;
@@ -128,6 +130,55 @@ sniff_include_count()
 	return srl1.nstrings;
 }
 
+/*
+ * NAME
+ *	sniff_use_this
+ *
+ * SYNOPSIS
+ *	void sniff_use_this(string_ty *path);
+ *
+ * DESCRIPTION
+ *	The sniff_use_this function is used to add file names
+ *	to a list of additional pathnames to use if a file can't
+ *	be found (probably because it needs to be generated)
+ *
+ * ARGUMENTS
+ *	path	- path to add
+ */
+
+void
+sniff_use_this(path)
+	char		*path;
+{
+	string_ty	*s;
+
+	assert(path);
+	trace(("sniff_use_this(path = \"%s\")\n{\n"/*}*/, path));
+	s = str_from_c(path);
+	string_list_append_unique(&use_these, s);
+	str_free(s);
+	trace((/*{*/"}\n"));
+}
+
+
+void
+sniff_use_this_cut()
+{
+	trace(("sniff_use_this_cut()\n{\n"/*}*/));
+	string_list_destructor(&use_these);
+	trace((/*{*/"}\n"));
+}
+
+
+long
+sniff_use_this_count()
+{
+	trace(("sniff_use_this_count()\n{\n"/*}*/));
+	trace(("return %ld;\n", (long)use_these.nstrings));
+	trace((/*{*/"}\n"));
+	return use_these.nstrings;
+}
+
 
 /*
  * NAME
@@ -345,6 +396,8 @@ resolve(filename, extra, srl, flags)
 		result = flatten(filename);
 		if (os_exists(result->str_text))
 			goto done;
+		if (string_list_member(&use_these, result))
+			goto done;
 		goto dilema;
 	}
 
@@ -367,6 +420,8 @@ resolve(filename, extra, srl, flags)
 		sub_context_delete(scp);
 		if (os_exists(result->str_text))
 			goto done;
+		if (string_list_member(&use_these, result))
+			goto done;
 		str_free(result);
 	}
 
@@ -388,6 +443,8 @@ resolve(filename, extra, srl, flags)
 		sub_context_delete(scp);
 		if (os_exists(result->str_text))
 			goto done;
+		if (string_list_member(&use_these, result))
+			goto done;
 		str_free(result);
 	}
 
@@ -549,6 +606,26 @@ print_without_prefix(s, result)
 }
 
 
+static void exclude_error _((string_ty *, string_ty *));
+
+static void
+exclude_error(includer, includee)
+	string_ty	*includer;
+	string_ty	*includee;
+{
+	sub_context_ty	*scp;
+
+	scp = sub_context_new();
+	sub_var_set(scp, "File_Name1", "%S", includer);
+	sub_var_set(scp, "File_Name2", "%S", includee);
+	fatal_intl
+	(
+		scp,
+		i18n("$filename1: excluded $filename2 referenced")
+	);
+}
+
+
 /*
  * NAME
  *	sniffer - search file for include dependencies
@@ -577,7 +654,7 @@ sniffer(filename, prnam, result)
 	int		prnam;
 	string_list_ty	*result;
 {
-	FILE		*fp;
+	input_ty	*fp;
 	cache_ty	*cp;
 	struct stat	st;
 	size_t		j;
@@ -621,10 +698,7 @@ sniffer(filename, prnam, result)
 			break;
 
 		default:
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", filename);
-			fatal_intl(scp, i18n("stat $filename: $errno"));
+			fatal_intl_stat(filename->str_text);
 			/* NOTREACHED */
 			break;
 		}
@@ -660,33 +734,14 @@ sniffer(filename, prnam, result)
 		verbose_intl(scp, i18n("cache miss for \"$filename\" file"));
 		sub_context_delete(scp);
 
-		fp = fopen(filename->str_text, "r");
-		if (!fp)
-		{
-			/*
-			 * probably "permission denied" error,
-			 * but file could have suddenly vanished
-			 */
+		if (!os_exists(filename->str_text))
 			goto absent;
-		}
+		fp = input_file_text_open(filename);
 		string_list_constructor(&type1);
 		string_list_constructor(&type2);
 		if (lang->scan(fp, &type1, &type2))
-		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", filename);
-			fatal_intl(scp, i18n("read $filename: $errno"));
-			/* NOTREACHED */
-		}
-		if (fclose(fp))
-		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", filename);
-			fatal_intl(scp, i18n("close $filename: $errno"));
-			/* NOTREACHED */
-		}
+			fatal_intl_read(filename->str_text);
+		input_delete(fp);
 
 		/*
 		 * type2 names have an implicit first element of the search path
@@ -737,6 +792,8 @@ sniffer(filename, prnam, result)
 				string_ty	*path;
 
 				path = type2.string[j];
+				if (string_list_member(&exclude, path))
+					exclude_error(filename, path);
 				if
 				(
 					!option.absolute
@@ -770,6 +827,8 @@ sniffer(filename, prnam, result)
 			string_ty	*path;
 
 			path = type1.string[j];
+			if (string_list_member(&exclude, path))
+				exclude_error(filename, path);
 			if
 			(
 				!option.absolute
@@ -897,14 +956,7 @@ sniff(filename)
 
 	if (option.output)
 	{
-		ofp = fopen(option.output, "w");
-		if (!ofp)
-		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", option.output);
-			fatal_intl(scp, i18n("open $filename: $errno"));
-		}
+		ofp = fopen_and_check(option.output, "w");
 		ofn = str_from_c(option.output);
 	}
 	else
@@ -972,20 +1024,9 @@ sniff(filename)
 	string_list_destructor(&result);
 
 	done:
-	if (fflush(ofp))
-	{
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", ofn);
-		fatal_intl(scp, i18n("write $filename: $errno"));
-	}
-	if (ofp != stdout && fclose(ofp))
-	{
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", ofn);
-		fatal_intl(scp, i18n("close $filename: $errno"));
-	}
+	fflush_and_check(ofp, ofn->str_text);
+	if (ofp != stdout)
+		fclose_and_check(ofp, ofn->str_text);
 	signal(SIGINT, int_hold);
 	signal(SIGHUP, hup_hold);
 	signal(SIGTERM, term_hold);
@@ -1055,7 +1096,7 @@ absolute_filename_test(path)
 
 void
 sniff_suffix_set(text)
-	char	*text;
+	char		*text;
 {
 	suffix = text;
 }
@@ -1063,7 +1104,7 @@ sniff_suffix_set(text)
 
 void
 sniff_prefix_set(text)
-	char	*text;
+	char		*text;
 {
 	string_ty	*s;
 
@@ -1071,3 +1112,15 @@ sniff_prefix_set(text)
 	string_list_append_unique(&prefix, s);
 	str_free(s);
 }
+
+
+void
+sniff_exclude(text)
+	char		*text;
+{
+	string_ty	*s;
+
+	s = str_from_c(text);
+	string_list_append_unique(&exclude, s);
+	str_free(s);
+}
