@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,6 @@
 #ifndef C_INCL_SNIFF_H
 #define C_INCL_SNIFF_H
 
-#include <stdio.h>
 #include <main.h>
 
 enum
@@ -48,12 +47,14 @@ struct option_ty
 };
 extern option_ty option;
 
+struct input_ty;
 struct string_list_ty;
 
 typedef struct sniff_ty sniff_ty;
 struct sniff_ty
 {
-	int (*scan)_((FILE *, struct string_list_ty *, struct string_list_ty *));
+	int (*scan)_((struct input_ty *, struct string_list_ty *,
+		struct string_list_ty *));
 	void (*prepare)_((void));
 };
 
@@ -61,6 +62,9 @@ void sniff _((char *));
 void sniff_include _((char *));
 void sniff_include_cut _((void));
 long sniff_include_count _((void));
+void sniff_use_this _((char *));
+void sniff_use_this_cut _((void));
+long sniff_use_this_count _((void));
 void sniff_prepare _((void));
 void sniff_language _((sniff_ty *));
 
@@ -71,5 +75,6 @@ int absolute_filename_test _((char *));
 
 void sniff_prefix_set _((char *));
 void sniff_suffix_set _((char *));
+void sniff_exclude _((char *));
 
 #endif /* C_INCL_SNIFF_H */
