@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997 Peter Miller;
+ *	Copyright (C) 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,15 +23,7 @@
 #ifndef COMMON_AC_STDIO_H
 #define COMMON_AC_STDIO_H
 
-/*
- * Need to define _POSIX_SOURCE on Linux, in order to get the fdopen,
- * fileno, popen and pclose function prototypes.
- */
-#ifdef __linux__
-#ifndef _POSIX_SOURCE
-#define _POSIX_SOURCE 1
-#endif
-#endif
+#include <config.h>
 
 #include <stdio.h>
 
@@ -42,4 +34,8 @@
 #define SEEK_SET 0
 #endif
 
+#ifndef L_tmpnam
+#define L_tmpnam 100
+#endif
+
 #endif /* COMMON_AC_STDIO_H */
