@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #include <ac/stddef.h>
 #include <ac/string.h>
 #include <ac/stdlib.h>
-#include <errno.h>
+#include <ac/errno.h>
 
 #include <error.h>
 #include <mem.h>
