@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997 Peter Miller;
+ *	Copyright (C) 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,10 @@
 #define COMMON_QUIT_H
 
 #include <main.h>
+#include <noreturn.h>
 
 typedef void (*quit_ty)_((void));
 void quit_handler _((quit_ty));
-void quit _((int));
+void quit _((int)) NORETURN;
 
 #endif /* COMMON_QUIT_H */
