@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -204,7 +204,7 @@ split()
 {
 	string_ty	*p;
 	string_ty	*p2;
-	str_hash_ty	index;
+	str_hash_ty	idx;
 
 	/*
 	 * get the list to be split across buckets 
@@ -240,12 +240,12 @@ split()
 		p2 = p;
 		p = p->str_next;
 
-		index = p2->str_hash & hash_cutover_mask;
-		if (index < hash_split)
-			index = p2->str_hash & hash_cutover_split_mask;
-		assert(index < hash_modulus);
-		p2->str_next = hash_table[index];
-		hash_table[index] = p2;
+		idx = p2->str_hash & hash_cutover_mask;
+		if (idx < hash_split)
+			idx = p2->str_hash & hash_cutover_split_mask;
+		assert(idx < hash_modulus);
+		p2->str_next = hash_table[idx];
+		hash_table[idx] = p2;
 	}
 }
 
@@ -302,7 +302,7 @@ str_n_from_c(s, length)
 	size_t		length;
 {
 	str_hash_ty	hash;
-	str_hash_ty	index;
+	str_hash_ty	idx;
 	string_ty	*p;
 
 	hash = hash_generate(s, length);
@@ -311,12 +311,12 @@ str_n_from_c(s, length)
 	if (!hash_table)
 		fatal_raw("you must call str_initialize early in main()");
 #endif
-	index = hash & hash_cutover_mask;
-	if (index < hash_split)
-		index = hash & hash_cutover_split_mask;
-	assert(index < hash_modulus);
+	idx = hash & hash_cutover_mask;
+	if (idx < hash_split)
+		idx = hash & hash_cutover_split_mask;
+	assert(idx < hash_modulus);
 
-	for (p = hash_table[index]; p; p = p->str_next)
+	for (p = hash_table[idx]; p; p = p->str_next)
 	{
 		if
 		(
@@ -336,8 +336,8 @@ str_n_from_c(s, length)
 	p->str_hash = hash;
 	p->str_length = length;
 	p->str_references = 1;
-	p->str_next = hash_table[index];
-	hash_table[index] = p;
+	p->str_next = hash_table[idx];
+	hash_table[idx] = p;
 #if 0
 	/* silence purify */
 	{
@@ -408,7 +408,7 @@ void
 str_free(s)
 	string_ty	*s;
 {
-	str_hash_ty	index;
+	str_hash_ty	idx;
 	string_ty	**spp;
 
 	assert(str_valid(s));
@@ -423,11 +423,11 @@ str_free(s)
 	 *  find the hash bucket it was in,
 	 *  and remove it
 	 */
-	index = s->str_hash & hash_cutover_mask;
-	if (index < hash_split)
-		index = s->str_hash & hash_cutover_split_mask;
-	assert(index < hash_modulus);
-	for (spp = &hash_table[index]; *spp; spp = &(*spp)->str_next)
+	idx = s->str_hash & hash_cutover_mask;
+	if (idx < hash_split)
+		idx = s->str_hash & hash_cutover_split_mask;
+	assert(idx < hash_modulus);
+	for (spp = &hash_table[idx]; *spp; spp = &(*spp)->str_next)
 	{
 		if (*spp == s)
 		{
@@ -442,97 +442,6 @@ str_free(s)
 }
 
 
-/*
- * NAME
- *	str_catenate - join two strings
- *
- * SYNOPSIS
- *	string_ty *str_catenate(string_ty *, string_ty *);
- *
- * DESCRIPTION
- *	The str_catenate function is used to concatenate two strings to form a
- *	new string.
- *
- * RETURNS
- *	string_ty * - a pointer to a string in dynamic memory.  Use str_free when
- *	finished with.
- *
- * CAVEAT
- *	The contents of the structure pointed to MUST NOT be altered.
- */
-
-string_ty *
-str_catenate(s1, s2)
-	string_ty	*s1;
-	string_ty	*s2;
-{
-	static char	*tmp;
-	static size_t	tmplen;
-	string_ty	*s;
-	size_t		length;
-
-	length = s1->str_length + s2->str_length;
-	if (tmplen < length)
-	{
-		tmplen = length;
-		tmp = mem_change_size(tmp, tmplen);
-	}
-	memcpy(tmp, s1->str_text, s1->str_length);
-	memcpy(tmp + s1->str_length, s2->str_text, s2->str_length);
-	s = str_n_from_c(tmp, length); 
-	return s;
-}
-
-
-/*
- * NAME
- *	str_cat_three - join three strings
- *
- * SYNOPSIS
- *	string_ty *str_cat_three(string_ty *, string_ty *, string_ty *);
- *
- * DESCRIPTION
- *	The str_cat_three function is used to concatenate three strings to form
- *	a new string.
- *
- * RETURNS
- *	string_ty * - a pointer to a string in dynamic memory.  Use str_free when
- *	finished with.
- *
- * CAVEAT
- *	The contents of the structure pointed to MUST NOT be altered.
- */
-
-string_ty *
-str_cat_three(s1, s2, s3)
-	string_ty	*s1;
-	string_ty	*s2;
-	string_ty	*s3;
-{
-	static char	*tmp;
-	static size_t	tmplen;
-	string_ty	*s;
-	size_t		length;
-
-	length = s1->str_length + s2->str_length + s3->str_length;
-	if (tmplen < length)
-	{
-		tmplen = length;
-		tmp = mem_change_size(tmp, tmplen);
-	}
-	memcpy(tmp, s1->str_text, s1->str_length);
-	memcpy(tmp + s1->str_length, s2->str_text, s2->str_length);
-	memcpy
-	(
-		tmp + s1->str_length + s2->str_length,
-		s3->str_text,
-		s3->str_length
-	);
-	s = str_n_from_c(tmp, length); 
-	return s;
-}
-
-
 /*
  * NAME
  *	str_equal - test equality of strings
@@ -564,102 +473,6 @@ str_equal(s1, s2)
 #endif
 
 
-/*
- * NAME
- *	str_upcase - upcase a string
- *
- * SYNOPSIS
- *	string_ty *str_upcase(string_ty *);
- *
- * DESCRIPTION
- *	The str_upcase function is used to form a string which is an upper case 
- *	form of the supplied string.
- *
- * RETURNS
- *	string_ty * - a pointer to a string in dynamic memory.  Use str_free when
- *	finished with.
- *
- * CAVEAT
- *	The contents of the structure pointed to MUST NOT be altered.
- */
-
-string_ty *
-str_upcase(s)
-	string_ty	*s;
-{
-	static char	*tmp;
-	static size_t	tmplen;
-	string_ty	*retval;
-	char		*cp1;
-	char		*cp2;
-
-	if (tmplen < s->str_length)
-	{
-		tmplen = s->str_length;
-		tmp = mem_change_size(tmp, tmplen);
-	}
-	for (cp1 = s->str_text, cp2 = tmp; *cp1; ++cp1, ++cp2)
-	{
-		int	c;
-
-		c = *cp1;
-		if (islower(c))
-			c = toupper(c);
-		*cp2 = c;
-	}
-	retval = str_n_from_c(tmp, s->str_length);
-	return retval;
-}
-
-
-/*
- * NAME
- *	str_downcase - lowercase string
- *
- * SYNOPSIS
- *	string_ty *str_downcase(string_ty *);
- *
- * DESCRIPTION
- *	The str_downcase function is used to form a string which is a lowercase
- *	form of the supplied string.
- *
- * RETURNS
- *	string_ty * - a pointer to a string in dynamic memory.  Use str_free when
- *	finished with.
- *
- * CAVEAT
- *	The contents of the structure pointed to MUST NOT be altered.
- */
-
-string_ty *
-str_downcase(s)
-	string_ty	*s;
-{
-	static char	*tmp;
-	static size_t	tmplen;
-	string_ty	*retval;
-	char		*cp1;
-	char		*cp2;
-
-	if (tmplen < s->str_length)
-	{
-		tmplen = s->str_length;
-		tmp = mem_change_size(tmp, tmplen);
-	}
-	for (cp1 = s->str_text, cp2 = tmp; *cp1; ++cp1, ++cp2)
-	{
-		int c;
-
-		c = *cp1;
-		if (isupper(c))
-			c = tolower(c);
-		*cp2 = c;
-	}
-	retval = str_n_from_c(tmp, s->str_length);
-	return retval;
-}
-
-
 /*
  * NAME
  *	str_bool - get boolean value
@@ -766,6 +579,8 @@ str_format(fmt sva_last)
 	result = vmprintfes(fmt, ap);
 	va_end(ap);
 	return result;
+	/* to silence debug warning */
+	trace(("bogus\n"));
 }
 
 
@@ -776,51 +591,3 @@ str_vformat(fmt, ap)
 {
 	return vmprintfes(fmt, ap);
 }
-
-
-string_ty *
-str_substitute(a, b, c)
-	string_ty	*a;
-	string_ty	*b;
-	string_ty	*c;
-{
-	char		*cp;
-	char		*ep;
-	static char	*buf;
-	static size_t	buf_max;
-	size_t		len;
-
-	cp = c->str_text;
-	ep = cp + c->str_length;
-	len = 0;
-	while (cp < ep)
-	{
-		if
-		(
-			ep - cp < a->str_length
-		||
-			0 != memcmp(a->str_text, cp, a->str_length)
-		)
-		{
-			if (len >= buf_max)
-			{
-				buf_max = buf_max * 2 + 16;
-				buf = mem_change_size(buf, buf_max);
-			}
-			buf[len++] = *cp++;
-		}
-		else
-		{
-			while (len + b->str_length > buf_max)
-			{
-				buf_max = buf_max * 2 + 16;
-				buf = mem_change_size(buf, buf_max);
-			}
-			memcpy(buf + len, b->str_text, b->str_length);
-			len += b->str_length;
-			cp += a->str_length;
-		}
-	}
-	return str_n_from_c(buf, len);
-	trace(("to silence warnings\n"));
-}
