@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  * MANIFEST: functions to perform $ substitutions
  */
 
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/libintl.h>
 #include <ac/limits.h>
 #include <ac/string.h>
@@ -1059,7 +1059,7 @@ subst_intl_wide(scp, msg)
 	language_human();
 	tmp = gettext(msg);
 	language_C();
-#ifdef DEBUG
+#if 0 /* def DEBUG */
 #ifdef HAVE_GETTEXT
 	if (tmp == msg)
 		error_raw("warning: message \"%s\" has no translation", msg);
