@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for common/symtab.c
  */
@@ -50,6 +50,7 @@ struct symtab_ty
 symtab_ty *symtab_alloc _((int));
 void symtab_free _((symtab_ty *));
 void *symtab_query _((symtab_ty *, string_ty *));
+void *symtab_query_fuzzy _((symtab_ty *, string_ty *, string_ty **));
 void symtab_assign _((symtab_ty *, string_ty *, void *));
 void symtab_assign_push _((symtab_ty *, string_ty *, void *));
 void symtab_delete _((symtab_ty *, string_ty *));
