@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997 Peter Miller;
+ *	Copyright (C) 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -163,7 +163,7 @@ split()
 {
 	wstring_ty	*p;
 	wstring_ty	*p2;
-	wstr_hash_ty	index;
+	wstr_hash_ty	idx;
 
 	/*
 	 * get the list to be split across buckets
@@ -195,11 +195,11 @@ split()
 		p2 = p;
 		p = p->wstr_next;
 
-		index = p2->wstr_hash & hash_cutover_mask;
-		if (index < hash_split)
-			index = p2->wstr_hash & hash_cutover_split_mask;
-		p2->wstr_next = hash_table[index];
-		hash_table[index] = p2;
+		idx = p2->wstr_hash & hash_cutover_mask;
+		if (idx < hash_split)
+			idx = p2->wstr_hash & hash_cutover_split_mask;
+		p2->wstr_next = hash_table[idx];
+		hash_table[idx] = p2;
 	}
 }
 
@@ -528,18 +528,18 @@ wstr_n_from_wc(s, length)
 	size_t		length;
 {
 	wstr_hash_ty	hash;
-	wstr_hash_ty	index;
+	wstr_hash_ty	idx;
 	wstring_ty	*p;
 
 	if (!hash_modulus)
 		wstr_initialize();
 	hash = hash_generate(s, length);
 
-	index = hash & hash_cutover_mask;
-	if (index < hash_split)
-		index = hash & hash_cutover_split_mask;
+	idx = hash & hash_cutover_mask;
+	if (idx < hash_split)
+		idx = hash & hash_cutover_split_mask;
 
-	for (p = hash_table[index]; p; p = p->wstr_next)
+	for (p = hash_table[idx]; p; p = p->wstr_next)
 	{
 		if
 		(
@@ -559,8 +559,8 @@ wstr_n_from_wc(s, length)
 	p->wstr_hash = hash;
 	p->wstr_length = length;
 	p->wstr_references = 1;
-	p->wstr_next = hash_table[index];
-	hash_table[index] = p;
+	p->wstr_next = hash_table[idx];
+	hash_table[idx] = p;
 	memcpy(p->wstr_text, s, length * sizeof(wchar_t));
 	p->wstr_text[length] = 0;
 
@@ -621,7 +621,7 @@ void
 wstr_free(s)
 	wstring_ty	*s;
 {
-	wstr_hash_ty	index;
+	wstr_hash_ty	idx;
 	wstring_ty	**spp;
 
 	if (!s)
@@ -637,10 +637,10 @@ wstr_free(s)
 	 * find the hash bucket it was in,
 	 * and remove it
 	 */
-	index = s->wstr_hash & hash_cutover_mask;
-	if (index < hash_split)
-		index = s->wstr_hash & hash_cutover_split_mask;
-	for (spp = &hash_table[index]; *spp; spp = &(*spp)->wstr_next)
+	idx = s->wstr_hash & hash_cutover_mask;
+	if (idx < hash_split)
+		idx = s->wstr_hash & hash_cutover_split_mask;
+	for (spp = &hash_table[idx]; *spp; spp = &(*spp)->wstr_next)
 	{
 		if (*spp == s)
 		{
