@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 
 #include <ac/ctype.h>
 #include <ac/stdio.h>
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/ar.h>
 #include <ac/fcntl.h>
 #include <ac/stdlib.h>
