@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -47,9 +47,11 @@
 #include <builtin/addsuffix.h>
 #include <builtin/basename.h>
 #include <builtin/boolean.h>
+#include <builtin/cando.h>
 #include <builtin/collect.h>
 #include <builtin/cook.h>
 #include <builtin/defined.h>
+#include <builtin/dos.h>
 #include <builtin/execute.h>
 #include <builtin/exists.h>
 #include <builtin/expr.h>
@@ -59,14 +61,19 @@
 #include <builtin/getenv.h>
 #include <builtin/glob.h>
 #include <builtin/home.h>
+#include <builtin/interi_files.h>
 #include <builtin/join.h>
 #include <builtin/match.h>
+#include <builtin/mtime.h>
 #include <builtin/opsys.h>
 #include <builtin/options.h>
 #include <builtin/pathname.h>
 #include <builtin/positional.h>
 #include <builtin/print.h>
 #include <builtin/readlink.h>
+#include <builtin/relati_dirna.h>
+#include <builtin/resolve.h>
+#include <builtin/sort_newest.h>
 #include <builtin/split.h>
 #include <builtin/stringset.h>
 #include <builtin/strip.h>
@@ -76,25 +83,14 @@
 #include <builtin/text.h>
 #include <builtin/thread-id.h>
 #include <builtin/unsplit.h>
+#include <builtin/uptodate.h>
 #include <builtin/word.h>
 #include <builtin/wordlist.h>
+#include <builtin/write.h>
 #include <id.h>
 #include <id/builtin.h>
 
 
-struct expr_position_ty; /* existence */
-
-
-typedef struct func_ty func_ty;
-struct func_ty
-{
-	char	*f_name;
-	int	(*f_code)_((string_list_ty *, const string_list_ty *,
-			const struct expr_position_ty *,
-			const struct opcode_context_ty *));
-};
-
-
 /*
  * NAME
  *	func - table of built-in functions
@@ -107,79 +103,89 @@ struct func_ty
  *	for the built-in functions of cook.
  */
 
-static func_ty func[] =
+static builtin_ty *func[] =
 {
-	{ "__FILE__",		builtin_FILE,		},
-	{ "__LINE__",		builtin_LINE,		},
-	{ "addprefix",		builtin_addprefix,	},
-	{ "addsuffix",		builtin_addsuffix,	},
-	{ "and",		builtin_and,		},
-	{ "basename",		builtin_basename,	},
-	{ "cando",		builtin_cando,		},
-	{ "catenate",		builtin_catenate,	},
-	{ "collect",		builtin_collect,	},
-	{ "collect_lines",	builtin_collect,	},
-	{ "count",		builtin_count,		},
-	{ "defined",		builtin_defined,	},
-	{ "dir",		builtin_dir,		},
-	{ "dirname",		builtin_dir,		},
-	{ "downcase",		builtin_downcase,	},
-	{ "entryname",		builtin_entryname,	},
-	{ "execute",		builtin_execute,	},
-	{ "exists",		builtin_exists,		},
-	{ "exists-symlink",	builtin_exists_symlink,	},
-	{ "expr",		builtin_expr,		},
-	{ "filter",		builtin_match_mask,	},
-	{ "filter-out",		builtin_filter_out,	},
-	{ "filter_out",		builtin_filter_out,	},
-	{ "find_command",	builtin_find_command,	},
-	{ "findstring",		builtin_findstring,	},
-	{ "firstword",		builtin_head,		},
-	{ "fromto",		builtin_fromto,		},
-	{ "getenv",		builtin_getenv,		},
-	{ "glob",		builtin_glob,		},
-	{ "head",		builtin_head,		},
-	{ "home",		builtin_home,		},
-	{ "if",			builtin_if,		},
-	{ "in",			builtin_in,		},
-	{ "join",		builtin_join,		},
-	{ "match",		builtin_match,		},
-	{ "match_mask",		builtin_match_mask,	},
-	{ "matches",		builtin_match,		},
-	{ "mtime",		builtin_mtime,		},
-	{ "not",		builtin_not,		},
-	{ "notdir",		builtin_entryname,	},
-	{ "operating_system",	builtin_opsys,		},
-	{ "options",		builtin_options,	},
-	{ "or",			builtin_or,		},
-	{ "os",			builtin_opsys,		},
-	{ "pathname",		builtin_pathname,	},
-	{ "patsubst",		builtin_fromto,		},
-	{ "prepost",		builtin_prepost,	},
-	{ "print",		builtin_print,	},
-	{ "quote",		builtin_quote,		},
-	{ "readlink",		builtin_readlink,	},
-	{ "resolve",		builtin_resolve,	},
-	{ "shell",		builtin_collect,	},
-	{ "sort",		builtin_sort,		},
-	{ "sort_newest",	builtin_sort_newest,	},
-	{ "split",		builtin_split,		},
-	{ "stringset",		builtin_stringset,	},
-	{ "strip",		builtin_strip,		},
-	{ "subst",		builtin_subst,		},
-	{ "substitute",		builtin_subst,		},
-	{ "substr",		builtin_substr,		},
-	{ "substring",		builtin_substr,		},
-	{ "suffix",		builtin_suffix,		},
-	{ "tail",		builtin_tail,		},
-	{ "thread-id",		builtin_thread_id,	},
-	{ "unsplit",		builtin_unsplit,	},
-	{ "upcase",		builtin_upcase,		},
-	{ "uptodate",		builtin_uptodate,	},
-	{ "wildcard",		builtin_glob,		},
-	{ "word",		builtin_word,		},
-	{ "wordlist",		builtin_wordlist,	},
-	{ "words",		builtin_count,		},
+	&builtin_FILE,
+	&builtin_LINE,
+	&builtin_addprefix,
+	&builtin_addsuffix,
+	&builtin_and,
+	&builtin_basename,
+	&builtin_cando,
+	&builtin_catenate,
+	&builtin_collect,
+	&builtin_collect_lines,
+	&builtin_cook,
+	&builtin_count,
+	&builtin_defined,
+	&builtin_dir,
+	&builtin_dirname,
+	&builtin_dos_path,
+	&builtin_dos_path_undo,
+	&builtin_downcase,
+	&builtin_entryname,
+	&builtin_execute,
+	&builtin_exists,
+	&builtin_exists_symlink,
+	&builtin_expr,
+	&builtin_filter,
+	&builtin_filter_out,
+	&builtin_filter_out_,
+	&builtin_find_command,
+	&builtin_findstring,
+	&builtin_firstword,
+	&builtin_fromto,
+	&builtin_getenv,
+	&builtin_glob,
+	&builtin_head,
+	&builtin_home,
+	&builtin_if,
+	&builtin_in,
+	&builtin_interior_files,
+	&builtin_join,
+	&builtin_leaf_files,
+	&builtin_match,
+	&builtin_match_mask,
+	&builtin_matches,
+	&builtin_mtime,
+	&builtin_mtime_seconds,
+	&builtin_not,
+	&builtin_notdir,
+	&builtin_operating_system,
+	&builtin_options,
+	&builtin_or,
+	&builtin_os,
+	&builtin_pathname,
+	&builtin_patsubst,
+	&builtin_prepost,
+	&builtin_print,
+	&builtin_quote,
+	&builtin_readlink,
+	&builtin_relative_dirname,
+	&builtin_reldir,
+	&builtin_resolve,
+	&builtin_shell,
+	&builtin_sort,
+	&builtin_sort_newest,
+	&builtin_split,
+	&builtin_stringset,
+	&builtin_strip,
+	&builtin_subst,
+	&builtin_substitute,
+	&builtin_substr,
+	&builtin_substring,
+	&builtin_suffix,
+	&builtin_tail,
+	&builtin_thread_id,
+	&builtin_un_dos_path,
+	&builtin_unsplit,
+	&builtin_upcase,
+	&builtin_uptodate,
+	&builtin_wildcard,
+	&builtin_word,
+	&builtin_wordlist,
+	&builtin_words,
+	&builtin_write,
 };
 
 
@@ -201,13 +207,13 @@ static func_ty func[] =
 void
 builtin_initialize()
 {
-	func_ty		*fp;
+	builtin_ty	**fpp;
 	string_ty	*s;
 
-	for (fp = func; fp < ENDOF(func); ++fp)
+	for (fpp = func; fpp < ENDOF(func); ++fpp)
 	{
-		s = str_from_c(fp->f_name);
-		id_assign(s, id_builtin_new(fp->f_code));
+		s = str_from_c((*fpp)->name);
+		id_assign(s, id_builtin_new(*fpp));
 		str_free(s);
 	}
 }
