@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -55,8 +55,11 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_defined(result, args, pp, ocp)
+static int interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -68,21 +71,6 @@ builtin_defined(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		if (id_search(args->string[j]))
@@ -92,3 +80,11 @@ builtin_defined(result, args, pp, ocp)
 	}
 	return 0;
 }
+
+
+builtin_ty builtin_defined =
+{
+	"defined",
+	interpret,
+	interpret, /* script */
+};
