@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -59,8 +59,11 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_execute(result, args, pp, ocp)
+static int interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -114,3 +117,11 @@ builtin_execute(result, args, pp, ocp)
 	string_list_append(result, s);
 	return 0;
 }
+
+
+builtin_ty builtin_execute =
+{
+	"execute",
+	interpret,
+	interpret, /* script */
+};
