@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -57,8 +57,11 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_exists(result, args, pp, ocp)
+static int exists_interpret _((string_list_ty *, const string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+exists_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const struct expr_position_ty *pp;
@@ -84,8 +87,19 @@ builtin_exists(result, args, pp, ocp)
 }
 
 
-int
-builtin_exists_symlink(result, args, pp, ocp)
+builtin_ty builtin_exists =
+{
+	"exists",
+	exists_interpret,
+	exists_interpret, /* script */
+};
+
+
+static int exists_symlink_interpret _((string_list_ty *, const string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+exists_symlink_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const struct expr_position_ty *pp;
@@ -109,3 +123,11 @@ builtin_exists_symlink(result, args, pp, ocp)
 	}
 	return 0;
 }
+
+
+builtin_ty builtin_exists_symlink =
+{
+	"exists-symlink",
+	exists_symlink_interpret,
+	exists_symlink_interpret, /* script */
+};
