@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,10 @@
  * MANIFEST: functions to perform shell-style file pattern matching
  */
 
+#include <ac/errno.h>
 #include <ac/stddef.h>
-#include <string.h>
-#include <errno.h>
 #include <ac/stdlib.h>
+#include <ac/string.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,45 +33,14 @@
 #include <error_intl.h>
 #include <expr/position.h>
 #include <mem.h>
+#include <stracc.h>
 #include <str_list.h>
 #include <trace.h>
 
 
-static	char	*original;
+static	char		*original;
 static	string_list_ty	*where;
-static	char	*tmp;
-static	size_t	tmp_max;
-static	size_t	tmp_len;
-
-
-/*
- * NAME
- *	tmp_char - append character
- *
- * SYNOPSIS
- *	void tmp_char(int c);
- *
- * DESCRIPTION
- *	The tmp_char function is used to append a character to the string being
- *	built by the builtin_glob function.
- *
- * RETURNS
- *	void
- */
-
-static void tmp_char _((int));
-
-static void
-tmp_char(c)
-	int		c;
-{
-	if (tmp_len >= tmp_max)
-	{
-		tmp_max += 128;
-		tmp = mem_change_size(tmp, tmp_max);
-	}
-	tmp[tmp_len++] = c;
-}
+static	stracc		temp;
 
 
 /*
@@ -372,7 +341,7 @@ globber(formal, pp)
 	for (;;)
 	{
 		while (*formal == '/')
-			tmp_char(*formal++);
+			sa_char(&temp, *formal++);
 		formal_end = strchr(formal, '/');
 		if (!formal_end)
 			formal_end = formal + strlen(formal);
@@ -387,22 +356,22 @@ globber(formal, pp)
 			/* nothing special */
 			trace(("ordinary = \"%.*s\";\n", formal_end - formal, formal));
 			for (cp = formal; cp < formal_end; ++cp)
-				tmp_char(*cp);
-			n = tmp_len;
-			tmp_char(0);
-			trace(("generated = \"%s\"\n", tmp));
+				sa_char(&temp, *cp);
+			n = sa_mark(&temp);
+			sa_char(&temp, 0);
+			trace(("generated = \"%s\"\n", temp.sa_buf));
 
 			/*
 			 * Need to confirm that it exists.  We are a
 			 * file name matcher, not a file name generator.
 			 */
-			if (stat(tmp, &st) < 0)
+			if (stat(temp.sa_buf, &st) < 0)
 			{
 				if (errno != ENOENT && errno != ENOTDIR)
 				{
 					scp = sub_context_new();
 					sub_errno_set(scp);
-					sub_var_set(scp, "File_Name", "%s", tmp);
+					sub_var_set(scp, "File_Name", "%s", temp.sa_buf);
 					error_with_position
 					(
 						pp,
@@ -417,13 +386,14 @@ globber(formal, pp)
 				/* fail the match */
 				break;
 			}
-			tmp_len = n;
+			sa_goto(&temp, n);
 
 			if (!*cp)
 			{
 				string_ty *s;
 
-				s = str_n_from_c(tmp, tmp_len);
+				s = sa_close(&temp);
+				temp.sa_inuse = 1; /* defeat assert protection */
 				string_list_append(where, s);
 				str_free(s);
 				break;
@@ -438,16 +408,16 @@ globber(formal, pp)
 
 			/* need to expand wild characters */
 			trace(("expand = \"%.*s\";\n", formal_end - formal, formal));
-			n = tmp_len;
-			tmp_char(0);
-			dp = opendir(tmp[0] ? tmp : ".");
+			n = sa_mark(&temp);
+			sa_char(&temp, 0);
+			dp = opendir(n ? temp.sa_buf : ".");
 			if (!dp)
 			{
 				if (errno == ENOTDIR)
 					break;
 				scp = sub_context_new();
 				sub_errno_set(scp);
-				sub_var_set(scp, "File_Name", "%s", tmp);
+				sub_var_set(scp, "File_Name", "%s", temp.sa_buf);
 				error_with_position
 				(
 					pp,
@@ -458,7 +428,7 @@ globber(formal, pp)
 				retval = -1;
 				goto ret;
 			}
-			tmp_len = n;
+			sa_goto(&temp, n);
 			for (;;)
 			{
 				char	*np;
@@ -489,18 +459,19 @@ globber(formal, pp)
 					goto ret;
 				}
 				for (cp = np; *cp; ++cp)
-					tmp_char(*cp);
+					sa_char(&temp, *cp);
 				if (!*formal_end)
 				{
 					string_ty *s;
 
-					s = str_n_from_c(tmp, tmp_len);
+					s = sa_close(&temp);
+					temp.sa_inuse = 1; /* defeat assert protection */
 					string_list_append(where, s);
 					str_free(s);
 				}
 				else
 				{
-					tmp_char('/');
+					sa_char(&temp, '/');
 					if (globber(formal_end + 1, pp))
 					{
 						closedir(dp);
@@ -508,7 +479,8 @@ globber(formal, pp)
 						goto ret;
 					}
 				}
-				tmp_len = n;
+				sa_goto(&temp, n);
+				temp.sa_inuse = 1;
 			}
 			closedir(dp);
 			break;
@@ -572,8 +544,11 @@ cmp(va, vb)
  *	This function is designed to be used as a "builtin" function.
  */
 
-int
-builtin_glob(result, args, pp, ocp)
+static int interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -588,7 +563,8 @@ builtin_glob(result, args, pp, ocp)
 	where = result;
 	for (j = 1; j < args->nstrings; ++j)
 	{
-		tmp_len = 0;
+		temp.sa_inuse = 0; /* bypass assert protection */
+		sa_open(&temp);
 		original = args->string[j]->str_text;
 		start = result->nstrings;
 		if (globber(original, pp))
@@ -608,3 +584,19 @@ builtin_glob(result, args, pp, ocp)
 	trace((/*{*/"}\n"));
 	return retval;
 }
+
+
+builtin_ty builtin_glob =
+{
+	"glob",
+	interpret,
+	interpret, /* script */
+};
+
+
+builtin_ty builtin_wildcard =
+{
+	"wildcard",
+	interpret,
+	interpret, /* script */
+};
