@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -46,6 +46,7 @@
 #include <error_intl.h>
 #include <expr/position.h>
 #include <match.h>
+#include <str_list.h>
 #include <trace.h>
 
 
@@ -73,15 +74,20 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_match(result, args, pp, ocp)
+static int match_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+match_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
 	const struct opcode_context_ty *ocp;
 {
-	int		j;
-	match_ty	*field;
+	size_t		j;
+	match_ty	*mp;
+	int		retval;
+	int		retval2;
 
 	trace(("match\n"));
 	assert(result);
@@ -102,23 +108,54 @@ builtin_match(result, args, pp, ocp)
 		sub_context_delete(scp);
 		return -1;
 	}
+	retval = 0;
+	mp = match_new();
+	if (match_compile(mp, args->string[1], pp) < 0)
+	{
+		retval = -1;
+		goto error_ret;
+	}
 	for (j = 2; j < args->nstrings; j++)
 	{
-		field = match(args->string[1], args->string[j], pp);
-		if (field == MATCH_ERROR)
-			return -1;
-		if (field)
+		retval2 = match_execute(mp, args->string[j], pp);
+		if (retval2 < 0)
 		{
-			string_list_append(result, str_true);
-			match_free(field);
+			retval = -1;
+			break;
+		}
+		if (retval2)
+		{
+			string_ty	*s;
+
+			s = str_format("%ld", (long)j - 1);
+			string_list_append(result, s);
+			str_free(s);
 		}
 		else
 			string_list_append(result, str_false);
 	}
-	return 0;
+	error_ret:
+	match_delete(mp);
+	return retval;
 }
 
 
+builtin_ty builtin_match =
+{
+	"match",
+	match_interpret,
+	match_interpret, /* script */
+};
+
+
+builtin_ty builtin_matches =
+{
+	"matches",
+	match_interpret,
+	match_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_match - wildcard mapping
@@ -143,15 +180,20 @@ builtin_match(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_match_mask(result, args, pp, ocp)
+static int match_mask_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+match_mask_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
 	const struct opcode_context_ty *ocp;
 {
-	int		j;
-	match_ty	*field;
+	size_t		j;
+	match_ty	*mp;
+	int		retval;
+	int		retval2;
 
 	trace(("match_mask\n"));
 	assert(result);
@@ -172,21 +214,46 @@ builtin_match_mask(result, args, pp, ocp)
 		sub_context_delete(scp);
 		return -1;
 	}
+	retval = 0;
+	mp = match_new();
+	if (match_compile(mp, args->string[1], pp) < 0)
+	{
+		retval = -1;
+		goto error_ret;
+	}
 	for (j = 2; j < args->nstrings; j++)
 	{
-		field = match(args->string[1], args->string[j], pp);
-		if (field == MATCH_ERROR)
-			return -1;
-		if (field)
+		retval2 = match_execute(mp, args->string[j], pp);
+		if (retval2 < 0)
 		{
-			string_list_append(result, args->string[j]);
-			match_free(field);
+			retval = -1;
+			break;
 		}
+		if (retval2)
+			string_list_append(result, args->string[j]);
 	}
-	return 0;
+	error_ret:
+	match_delete(mp);
+	return retval;
 }
 
 
+builtin_ty builtin_match_mask =
+{
+	"match_mask",
+	match_mask_interpret,
+	match_mask_interpret, /* script */
+};
+
+
+builtin_ty builtin_filter =
+{
+	"filter",
+	match_mask_interpret,
+	match_mask_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_fromto - wildcard mapping
@@ -211,15 +278,20 @@ builtin_match_mask(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_fromto(result, args, pp, ocp)
+static int fromto_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+fromto_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
 	const struct opcode_context_ty *ocp;
 {
-	int		j;
-	match_ty	*field;
+	size_t		j;
+	match_ty	*mp;
+	int		retval;
+	int		retval2;
 
 	trace(("fromto\n"));
 	assert(result);
@@ -240,24 +312,54 @@ builtin_fromto(result, args, pp, ocp)
 		sub_context_delete(scp);
 		return -1;
 	}
+	retval = 0;
+	mp = match_new();
+	if (match_compile(mp, args->string[1], pp) < 0)
+	{
+		retval = -1;
+		goto error_ret;
+	}
 	for (j = 3; j < args->nstrings; j++)
 	{
-		field = match(args->string[1], args->string[j], pp);
-		if (field == MATCH_ERROR)
-			return -1;
-		if (field)
+		retval2 = match_execute(mp, args->string[j], pp);
+		if (retval2 < 0)
+		{
+			retval = -1;
+			break;
+		}
+		if (retval2)
 		{
 			string_ty	*s;
 
-			s = reconstruct(args->string[2], field, pp);
-			match_free(field);
+			s = match_reconstruct_rhs(mp, args->string[2], pp);
 			if (!s)
-				return -1;
+			{
+				retval = -1;
+				break;
+			}
 			string_list_append(result, s);
 			str_free(s);
 		}
 		else
 			string_list_append(result, args->string[j]);
 	}
-	return 0;
+	error_ret:
+	match_delete(mp);
+	return retval;
 }
+
+
+builtin_ty builtin_fromto =
+{
+	"fromto",
+	fromto_interpret,
+	fromto_interpret, /* script */
+};
+
+
+builtin_ty builtin_patsubst =
+{
+	"patsubst",
+	fromto_interpret,
+	fromto_interpret, /* script */
+};
