@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -55,8 +55,11 @@
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_dir(result, args, pp, ocp)
+static int dir_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+dir_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -68,21 +71,6 @@ builtin_dir(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
@@ -97,6 +85,22 @@ builtin_dir(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_dir =
+{
+	"dir",
+	dir_interpret,
+	dir_interpret, /* script */
+};
+
+
+builtin_ty builtin_dirname =
+{
+	"dirname",
+	dir_interpret,
+	dir_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_entryname - entryname part
@@ -119,8 +123,11 @@ builtin_dir(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_entryname(result, args, pp, ocp)
+static int entryname_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+entryname_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -132,21 +139,6 @@ builtin_entryname(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
@@ -161,6 +153,22 @@ builtin_entryname(result, args, pp, ocp)
 }
 
 
+builtin_ty builtin_entryname =
+{
+	"entryname",
+	entryname_interpret,
+	entryname_interpret, /* script */
+};
+
+
+builtin_ty builtin_notdir =
+{
+	"notdir",
+	entryname_interpret,
+	entryname_interpret, /* script */
+};
+
+
 /*
  * NAME
  *	builtin_pathname - pathname part
@@ -183,8 +191,11 @@ builtin_entryname(result, args, pp, ocp)
  *	the result when it is finished, with a string_list_destructor() call.
  */
 
-int
-builtin_pathname(result, args, pp, ocp)
+static int pathname_interpret _((string_list_ty *, const string_list_ty *,
+	const expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+pathname_interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const expr_position_ty *pp;
@@ -196,21 +207,6 @@ builtin_pathname(result, args, pp, ocp)
 	assert(result);
 	assert(args);
 	assert(args->nstrings);
-	if (args->nstrings < 2)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_var_set(scp, "Name", "%S", args->string[0]);
-		error_with_position
-		(
-			pp,
-			scp,
-			i18n("$name: requires one or more arguments")
-		);
-		sub_context_delete(scp);
-		return -1;
-	}
 	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
@@ -223,3 +219,11 @@ builtin_pathname(result, args, pp, ocp)
 	}
 	return 0;
 }
+
+
+builtin_ty builtin_pathname =
+{
+	"pathname",
+	pathname_interpret,
+	pathname_interpret, /* script */
+};
