@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -141,8 +141,11 @@ stringset_one(result)
 }
 
 
-int
-builtin_stringset(result, args, pp, ocp)
+static int interpret _((string_list_ty *, const string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+
+static int
+interpret(result, args, pp, ocp)
 	string_list_ty	*result;
 	const string_list_ty *args;
 	const struct expr_position_ty *pp;
@@ -161,3 +164,11 @@ builtin_stringset(result, args, pp, ocp)
 	stringset_args = 0;
 	return 0;
 }
+
+
+builtin_ty builtin_stringset =
+{
+	"stringset",
+	interpret,
+	interpret, /* script */
+};
