@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -79,7 +79,7 @@
  */
 
 #include <ac/stddef.h>
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/time.h>
 
 #include <cascade.h>
@@ -89,16 +89,18 @@
 #include <error_intl.h>
 #include <expr.h>
 #include <flag.h>
-#include <fngrprnt.h>
+#include <fingerprint.h>
+#include <fingerprint/value.h>
 #include <graph.h>
 #include <graph/build.h>
 #include <graph/file_pair.h>
 #include <graph/leaf.h>
 #include <graph/stats.h>
 #include <graph/walk.h>
+#include <graph/web.h>
 #include <id.h>
 #include <id/variable.h>
-#include <match.h>
+#include <match/new_by_recip.h>
 #include <mem.h>
 #include <option.h>
 #include <os.h>
@@ -213,14 +215,15 @@ cook_search_list(slp)
  */
 
 time_t
-cook_mtime_oldest(path, depth_p)
+cook_mtime_oldest(path, depth_p, max_fp_depth)
 	string_ty	*path;
 	long		*depth_p;
+	long		max_fp_depth;
 {
 	time_t		result;
 	long		bogus;
 
-	trace(("cook_mtime_oldest(path = \"%s\")\n{\n"/*}*/, path->str_text));
+	trace(("cook_mtime_oldest(path = \"%s\", max = %ld)\n{\n"/*}*/, path->str_text, max_fp_depth));
 	if (!depth_p)
 		depth_p = &bogus;
 	if (path->str_text[0] == '/')
@@ -230,7 +233,7 @@ cook_mtime_oldest(path, depth_p)
 	}
 	else
 	{
-		fp_ty		*prv_fp;
+		fp_value_ty	*prv_fp;
 		string_list_ty	sl;
 		long		j;
 
@@ -240,7 +243,7 @@ cook_mtime_oldest(path, depth_p)
 		*depth_p = sl.nstrings;
 		for (j = 0; j < sl.nstrings; ++j)
 		{
-			fp_ty		*fp;
+			fp_value_ty	*fp;
 			string_ty	*s1;
 			string_ty	*s2;
 			time_t		t;
@@ -273,7 +276,6 @@ cook_mtime_oldest(path, depth_p)
 				result = t;
 				*depth_p = j;
 				trace(("is the first\n"));
-
 				if (option_test(OPTION_FINGERPRINT))
 				{
 					prv_fp = fp_search(s2);
@@ -290,6 +292,11 @@ cook_mtime_oldest(path, depth_p)
 
 			/* Found a deeper version */
 			assert(option_test(OPTION_FINGERPRINT));
+			if (j >= max_fp_depth)
+			{
+				str_free(s2);
+				break;
+			}
 			fp = fp_search(s2);
 			str_free(s2);
 
@@ -346,9 +353,10 @@ cook_mtime_oldest(path, depth_p)
  */
 
 time_t
-cook_mtime_newest(path, depth_p)
+cook_mtime_newest(path, depth_p, max_fp_depth)
 	string_ty	*path;
 	long		*depth_p;
+	long		max_fp_depth;
 {
 	time_t		result;
 
@@ -360,7 +368,7 @@ cook_mtime_newest(path, depth_p)
 	}
 	else
 	{
-		fp_ty		*prv_fp;
+		fp_value_ty	*prv_fp;
 		string_list_ty	sl;
 		long		j;
 
@@ -370,7 +378,7 @@ cook_mtime_newest(path, depth_p)
 		*depth_p = sl.nstrings;
 		for (j = 0; j < sl.nstrings; ++j)
 		{
-			fp_ty		*fp;
+			fp_value_ty	*fp;
 			string_ty	*s1;
 			string_ty	*s2;
 			time_t		t;
@@ -420,6 +428,11 @@ cook_mtime_newest(path, depth_p)
 
 			/* Found a deeper version */
 			assert(option_test(OPTION_FINGERPRINT));
+			if (j >= max_fp_depth)
+			{
+				str_free(s2);
+				break;
+			}
 			fp = fp_search(s2);
 			str_free(s2);
 
@@ -591,6 +604,8 @@ cook(wlp)
 		option_test(OPTION_CASCADE)
 	&&
 		!option_test(OPTION_SILENT)
+	&&
+		option_test(OPTION_INCLUDE_COOKED_WARNING)
 	)
 	{
 		gp->file_pair = graph_file_pair_new((string_list_ty *)0);
@@ -752,10 +767,10 @@ cook_pairs(wlp)
 
 /*
  * NAME
- *	fubar
+ *	cook_script
  *
  * SYNOPSIS
- *	void fubar(void);
+ *	void cook_script(string_list_ty *);
  *
  * DESCRIPTION
  *	The cook_script function is used to print a shell script to
@@ -847,29 +862,23 @@ cook_script(wlp)
 
 /*
  * NAME
- *	isit_uptodate - test construct files
+ *	cook_web
  *
  * SYNOPSIS
- *	int isit_uptodate(string_ty *target);
+ *	void cook_web(string_list_ty *);
  *
  * DESCRIPTION
- *	The isit_uptodate function is used test if the giben target
- *	is up to date.
+ *	The cook_web function is used to print a shell web to
+ *	build the the given targets.  It's only an approximation of the
+ *	full cook semantics.
  *
  * RETURNS
- *	The isit_uptodate function returns
- *		-1 if any error is encountered
- *		0 if not up to date, or don't know how
- *		1 if target is uptodate.
- *
- * CAVEAT
- *	This function must be called after everything has been initialized,
- *	and the cookbook read in.
+ *	int; 0 on success, 1 on failure (exit statii)
  */
 
 int
-isit_uptodate(target)
-	string_ty	*target;
+cook_web(wlp)
+	string_list_ty	*wlp;
 {
 	int		retval;
 	graph_ty	*gp;
@@ -881,8 +890,7 @@ isit_uptodate(target)
 	 * Note that tee(1) [see listing.c] must ignore them
 	 * for the generated messages to appear in the log file.
 	 */
-	trace(("isit_uptodate(target = %08lX)\n{\n"/*}*/, target));
-	trace_string(target->str_text);
+	trace(("cook(wlp = %08lX)\n{\n", wlp));
 	desist_enable();
 
 	/*
@@ -890,32 +898,32 @@ isit_uptodate(target)
 	 */
 	retval = 0;
 	gp = graph_new();
-	gb_status = graph_build(gp, target, graph_build_preference_backtrack, 0);
+	gb_status = graph_build_list(gp, wlp, graph_build_preference_error, 0);
 	if (option_test(OPTION_REASON))
 		graph_print_statistics(gp);
 	switch (gb_status)
 	{
 	case graph_build_status_error:
-		retval = -1;
+		retval = 1;
 		break;
 
 	case graph_build_status_backtrack:
-		retval = 0;
+		/* assert(0); */
+		retval = 1;
 		break;
 
 	case graph_build_status_success:
-		retval = 1;
 		break;
 	}
 
 	/*
 	 * Walk the dependency graph.
 	 */
-	if (retval > 0)
-		retval = graph_isit_uptodate(gp);
+	if (retval == 0)
+		graph_walk_web(gp);
 
 	/*
-	 * Release resources held by the graph.
+	 * Release any resources held by the graph.
 	 */
 	graph_delete(gp);
 
@@ -923,88 +931,7 @@ isit_uptodate(target)
 	 * Return the result to the caller.
 	 */
 	trace(("return %d;\n", retval));
-	trace((/*{*/"}\n"));
-	return retval;
-}
-
-
-/*
- * NAME
- *	cando - test construct files
- *
- * SYNOPSIS
- *	int cando(string_ty *target);
- *
- * DESCRIPTION
- *	The cando function is used to test if the given target is cookable.
- *
- * RETURNS
- *	The cando function returns
- *		-1 if any error was encountered
- *		0 if can't cook the target
- *		1 if can cook target
- *
- * CAVEAT
- *	This function must be called after everything has been initialized,
- *	and the cookbook read in.
- */
-
-int
-cando(target)
-	string_ty	*target;
-{
-	int		retval;
-	graph_ty	*gp;
-	graph_build_status_ty gb_status;
-
-	/*
-	 * set interrupts to catch
-	 *
-	 * Note that tee(1) [see listing.c] must ignore them
-	 * for the generated messages to appear in the log file.
-	 */
-	trace(("cando(target = %08lX)\n{\n"/*}*/, target));
-	trace_string(target->str_text);
-	desist_enable();
-
-	/*
-	 * Build the dependency graph.
-	 */
-	retval = 0;
-	gp = graph_new();
-	gb_status = graph_build(gp, target, graph_build_preference_backtrack, 0);
-	if (option_test(OPTION_REASON))
-		graph_print_statistics(gp);
-
-	/*
-	 * it is only relevant that we know how to build this graph, not
-	 * that we walk it.
-	 */
-	switch (gb_status)
-	{
-	case graph_build_status_error:
-		retval = -1;
-		break;
-
-	case graph_build_status_backtrack:
-		retval = 0;
-		break;
-
-	case graph_build_status_success:
-		retval = 1;
-		break;
-	}
-
-	/*
-	 * Release resources held by the graph.
-	 */
-	graph_delete(gp);
-
-	/*
-	 * Return the result to the caller.
-	 */
-	trace(("return %d;\n", retval));
-	trace((/*{*/"}\n"));
+	trace(("}\n"));
 	return retval;
 }
 
@@ -1075,7 +1002,12 @@ cook_auto_required()
 	 * Build the dependency graph.
 	 */
 	gp = graph_new();
-	if (!option_test(OPTION_SILENT))
+	if
+	(
+		!option_test(OPTION_SILENT)
+	&&
+		option_test(OPTION_INCLUDE_COOKED_WARNING)
+	)
 		gp->file_pair = graph_file_pair_new(&cook_auto_list);
 	gb_status =
 		graph_build_list
@@ -1361,13 +1293,21 @@ cook_implicit_append(rp)
 	string_list_ty	base_list;
 	recipe_list_ty	*rlp;
 	size_t		j;
+	match_ty	*mp;
+
+	/*
+	 * Create a suitable matching object.  We need to set the recipe
+	 * flags, to know which matching flavour.
+	 */
+	mp = match_new_by_recipe(rp);
 
 	string_list_constructor(&base_list);
 	for (j = 0; j < rp->target->nstrings; ++j)
 	{
 		base = os_entryname(rp->target->string[j]);
-		if (match_usage_mask(base) != 0)
+		if (match_usage_mask(mp, base, &rp->pos) != 0)
 		{
+			match_delete(mp);
 			str_free(base);
 			string_list_destructor(&base_list);
 			recipe_list_append(&implicit, rp);
@@ -1376,6 +1316,7 @@ cook_implicit_append(rp)
 		string_list_append_unique(&base_list, base);
 		str_free(base);
 	}
+	match_delete(mp);
 	for (j = 0; j < base_list.nstrings; ++j)
 	{
 		base = base_list.string[j];
