@@ -1,7 +1,7 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997
- *	Peter Miller; All rights reserved.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1999 Peter Miller;
+ *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -34,11 +34,10 @@ struct string_list_ty; /* existence */
 int cook _((struct string_list_ty *));
 int cook_pairs _((struct string_list_ty *));
 int cook_script _((struct string_list_ty *));
+int cook_web _((struct string_list_ty *));
 
-int isit_uptodate _((struct string_ty *));
-int cando _((struct string_ty *));
-time_t cook_mtime_oldest _((struct string_ty *path, long *depth_p));
-time_t cook_mtime_newest _((struct string_ty *path, long *depth_p));
+time_t cook_mtime_oldest _((struct string_ty *, long *, long));
+time_t cook_mtime_newest _((struct string_ty *, long *, long));
 int cook_mtime_resolve _((struct string_list_ty *, const struct string_list_ty *, int));
 
 void cook_auto _((struct string_list_ty *));
