@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <desist.h>
 #include <error_intl.h>
 #include <expr.h>
+#include <fingerprint/sync.h>
 #include <graph.h>
 #include <graph/build.h>
 #include <graph/file.h>
@@ -35,7 +36,8 @@
 #include <graph/recipe_list.h>
 #include <id.h>
 #include <id/variable.h>
-#include <match.h>
+#include <match/new_by_recip.h>
+#include <match/wl.h>
 #include <opcode/context.h>
 #include <opcode/list.h>
 #include <option.h>
@@ -188,7 +190,7 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	string_list_constructor(target);
 	if (mp)
 	{
-		if (wl_reconstruct(target, rp->target, mp, &rp->pos) < 0)
+		if (match_wl_reconstruct_lhs(mp, target, rp->target, &rp->pos) < 0)
 		{
 			trace(("return error;\n"));
 			trace((/*{*/"}\n"));
@@ -244,6 +246,43 @@ graph_check_ingredients(gp, rp, preference, target, need_gfl,
 	implicit_allowed = option_test(OPTION_IMPLICIT_ALLOWED);
 	option_undo(OPTION_IMPLICIT_ALLOWED, OPTION_LEVEL_DEFAULT);
 
+	/*
+	 * If there is a precondition and if the gatefirst option is set
+	 * then check the precondition before evaluating the ingredients.
+	 * If the precondition fails then skip out of this recipe
+	 * right now...
+	 */
+	trace(("check gatefirst precondition\n"));
+	if (rp->precondition && option_test(OPTION_GATEFIRST))
+	{
+		int		flag;
+		flag = opcode_list_run_bool(rp->precondition, mp);
+
+		switch (flag)
+		{
+		case -1:
+			gp->statistic.error_in_expr++;
+                        status = graph_build_status_error;
+                        goto done;
+
+		case 0:
+			if (option_test(OPTION_REASON))
+			{
+				scp = sub_context_new();
+				sub_var_set(scp, "File_Name", "%S", target1);
+				error_with_position
+				(
+					&rp->pos,
+					scp,
+			     i18n("\"$filename\" precondition rejects (reason)")
+				);
+				sub_context_delete(scp);
+			}
+			gp->statistic.precondition_rejection++;
+			goto backtrack;
+		}
+	}
+
 	/*
 	 * wander through the ingredients, seeing if we know how to cook them
 	 *	the first set is used to determine if to use this recipe
@@ -317,7 +356,8 @@ backtrack:
 		}
 		if (mp && !option_test(OPTION_RECURSE))
 		{
-			match_ty	*inhibit;
+			match_ty	*mp2;
+			int		inhibit;
 
 			/*
 			 * if any of the ingredients of an
@@ -325,14 +365,16 @@ backtrack:
 			 * that same recipe, inhibit the
 			 * recursive application the recipe.
 			 */
-			inhibit = wl_match(rp->target, target2, &rp->pos);
-			if (inhibit == MATCH_ERROR)
+			trace(("mark\n"));
+			mp2 = match_clone(mp);
+			inhibit = match_wl_attempt(mp2, rp->target, target2, &rp->pos);
+			match_delete(mp2);
+			if (inhibit < 0)
 				goto error;
 			if (inhibit)
 			{
 				rp->inhibit = 1;
 				gp->statistic.inhibit_self_recursion++;
-				match_free(inhibit);
 			}
 		}
 		option_undo_level(OPTION_LEVEL_RECIPE);
@@ -387,6 +429,7 @@ backtrack:
 		string_list_delete(wlp1);
 		goto error;
 	}
+	trace(("mark\n"));
 
 	/*
 	 * remove "./" for the start of file names,
@@ -422,7 +465,8 @@ backtrack:
 		}
 		if (mp && !option_test(OPTION_RECURSE))
 		{
-			match_ty	*inhibit;
+			match_ty	*mp2;
+			int		inhibit;
 
 			/*
 			 * if any of the ingredients of an
@@ -430,14 +474,16 @@ backtrack:
 			 * that same recipe, inhibit the
 			 * recursive application the recipe.
 			 */
-			inhibit = wl_match(rp->target, target2, &rp->pos);
-			if (inhibit == MATCH_ERROR)
+			trace(("mark\n"));
+			mp2 = match_clone(mp);
+			inhibit = match_wl_attempt(mp2, rp->target, target2, &rp->pos);
+			match_delete(mp2);
+			if (inhibit < 0)
 				goto error;
 			if (inhibit)
 			{
 				rp->inhibit = 1;
 				gp->statistic.inhibit_self_recursion++;
-				match_free(inhibit);
 			}
 		}
 		option_undo_level(OPTION_LEVEL_RECIPE);
@@ -498,6 +544,7 @@ backtrack:
 	 * there.  (They seem to read the recipe left-to-right, and
 	 * expect the common ingredients on the end as ``extra''.)
 	 */
+	trace(("mark\n"));
 	if (common_ingredients_gfl)
 	{
 		for (j = 0; j < common_ingredients_gfl->nfiles; ++j)
@@ -514,6 +561,7 @@ backtrack:
 	/*
 	 * Find the cascaded ingredients, and add them.
 	 */
+	trace(("mark\n"));
 	if (cascade_enabled)
 		cascade_find(&need, &cascade);
 	for (k = 0; k < cascade.length; ++k)
@@ -548,7 +596,8 @@ backtrack:
 		}
 		if (mp && !option_test(OPTION_RECURSE))
 		{
-			match_ty	*inhibit;
+			match_ty	*mp2;
+			int		inhibit;
 
 			/*
 			 * if any of the ingredients of an
@@ -556,14 +605,15 @@ backtrack:
 			 * that same recipe, inhibit the
 			 * recursive application the recipe.
 			 */
-			inhibit = wl_match(rp->target, target2, &rp->pos);
-			if (inhibit == MATCH_ERROR)
+			mp2 = match_clone(mp);
+			inhibit = match_wl_attempt(mp2, rp->target, target2, &rp->pos);
+			match_delete(mp2);
+			if (inhibit < 0)
 				goto error;
 			if (inhibit)
 			{
 				rp->inhibit = 1;
 				gp->statistic.inhibit_self_recursion++;
-				match_free(inhibit);
 			}
 		}
 		option_undo_level(OPTION_LEVEL_RECIPE);
@@ -615,10 +665,13 @@ backtrack:
 	}
 
 	/*
-	 * see if this recipe applies
+	 * If there is a precondition and if the gatefirst option is
+	 * not set then check the precondition, now that we have evaluated
+	 * the ingredients.  If the precondition fails then skip out of this
+	 * recipe right now...
 	 */
 	trace(("check precondition\n"));
-	if (rp->precondition)
+	if (rp->precondition && !option_test(OPTION_GATEFIRST))
 	{
 		int		flag;
 
@@ -773,7 +826,7 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	string_list_ty	target;
 	size_t		k;
 
-	trace(("graph_check_recipe(gp = %8.8lX, rp = %8.8lX, preference = %d)\n{\n"/*}*/,
+	trace(("graph_check_recipe(gp = %8.8lX, rp = %8.8lX, preference = %d)\n{\n",
 		(long)gp, (long)rp, (int)preference));
 	status =
 		graph_check_ingredients
@@ -799,8 +852,10 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 		string_list_destructor(&target);
 		graph_file_list_nrc_destructor(&need_gfl);
 		 */
+		if (mp)
+			match_delete(mp);
 		trace(("return %s;\n", graph_build_status_name(status)));
-		trace((/*{*/"}\n"));
+		trace(("}\n"));
 		return status;
 
 	case graph_build_status_success:
@@ -826,7 +881,9 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	if (rp->single_thread)
 	{
 		grp->single_thread = opcode_list_run(rp->single_thread, mp);
-		if (grp->single_thread->nstrings == 0)
+		if (!grp->single_thread)
+			status = graph_build_status_error;
+		else if (grp->single_thread->nstrings == 0)
 		{
 			string_list_delete(grp->single_thread);
 			grp->single_thread = 0;
@@ -839,7 +896,9 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	if (rp->host_binding)
 	{
 		grp->host_binding = opcode_list_run(rp->host_binding, mp);
-		if (grp->host_binding->nstrings == 0)
+		if (!grp->host_binding)
+			status = graph_build_status_error;
+		else if (grp->host_binding->nstrings == 0)
 		{
 			string_list_delete(grp->host_binding);
 			grp->host_binding = 0;
@@ -886,11 +945,11 @@ graph_check_recipe(gp, rp, preference, common_ingredients_gfl, implicit_allowed,
 	graph_file_list_nrc_destructor(&need_gfl);
 
 	/*
-	 * success
+	 * return result
 	 */
-	trace(("return success;\n"));
-	trace((/*{*/"}\n"));
-	return graph_build_status_success;
+	trace(("return %s;\n", graph_build_status_name(status)));
+	trace(("}\n"));
+	return status;
 }
 
 
@@ -1029,6 +1088,7 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 		trace((/*{*/"}\n"));
 		return;
 	}
+	fp_sync();
 
 	/*
 	 * Check to see if this one has been cooked already.
@@ -1235,6 +1295,7 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			match_ty	*mp;
 			string_list_ty	target2;
 			graph_file_list_nrc_ty need2_gfl;
+			int		ok;
 
 			rp = cook_implicit_nth_by_name(j, base);
 			if (!rp)
@@ -1242,11 +1303,25 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			if (rp->out_of_date)
 				continue;
 			gp->statistic.pattern_match_query++;
-			mp = wl_match(rp->target, target, &rp->pos);
-			if (mp == MATCH_ERROR)
+
+			/*
+			 * new matcher per recipe, since each one can
+			 * have a different matcher
+			 */
+			trace(("mark\n"));
+			mp = match_new_by_recipe(rp);
+
+			ok = match_wl_attempt(mp, rp->target, target, &rp->pos);
+			if (ok < 0)
+			{
+				match_delete(mp);
 				goto bomb_with_error;
-			if (!mp)
+			}
+			if (!ok)
+			{
+				match_delete(mp);
 				continue;
+			}
 
 			status2 =
 				graph_check_ingredients
@@ -1260,7 +1335,8 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 					implicit_allowed,
 					mp
 				);
-			match_free(mp);
+			match_delete(mp);
+			mp = 0;
 
 			switch (status2)
 			{
@@ -1295,6 +1371,7 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			match_ty	*mp;
 			string_list_ty	target2;
 			graph_file_list_nrc_ty need2_gfl;
+			int		ok;
 
 			rp = cook_implicit_nth(j);
 			if (!rp)
@@ -1302,11 +1379,25 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			if (rp->out_of_date)
 				continue;
 			gp->statistic.pattern_match_query++;
-			mp = wl_match(rp->target, target, &rp->pos);
-			if (mp == MATCH_ERROR)
+
+			/*
+			 * new matcher per recipe, since each one can
+			 * have a different matcher
+			 */
+			trace(("mark\n"));
+			mp = match_new_by_recipe(rp);
+
+			ok = match_wl_attempt(mp, rp->target, target, &rp->pos);
+			if (ok < 0)
+			{
+				match_delete(mp);
 				goto bomb_with_error;
-			if (!mp)
+			}
+			if (!ok)
+			{
+				match_delete(mp);
 				continue;
+			}
 
 			status2 =
 				graph_check_ingredients
@@ -1320,7 +1411,8 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 					implicit_allowed,
 					mp
 				);
-			match_free(mp);
+			match_delete(mp);
+			mp = 0;
 
 			switch (status2)
 			{
@@ -1453,13 +1545,27 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			for (k = 0; k < rp->target->nstrings; ++k)
 			{
 				string_ty	*target_pattern;
+				int		ok;
+
+				/*
+				 * new matcher per recipe, since each one can
+				 * have a different matcher
+				 */
+				trace(("mark\n"));
+				mp = match_new_by_recipe(rp);
 
 				target_pattern = rp->target->string[k];
-				mp = match(target_pattern, target, &rp->pos);
-				if (mp == MATCH_ERROR)
+				ok = match_attempt(mp, target_pattern, target, &rp->pos);
+				if (ok < 0)
+				{
+					match_delete(mp);
 					goto bomb_with_error;
-				if (!mp)
+				}
+				if (!ok)
+				{
+					match_delete(mp);
 					continue;
+				}
 
 				status2 =
 					graph_check_recipe
@@ -1471,16 +1577,15 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 						implicit_allowed,
 						mp
 					);
+				mp = 0;
 
 				switch (status2)
 				{
 				case graph_build_status_error:
-					match_free(mp);
 					trace(("child build error\n"));
 					goto bomb_with_error;
 
 				case graph_build_status_backtrack:
-					match_free(mp);
 					trace(("child build backtrack\n"));
 					gp->statistic.implicit_not_applicable++;
 					continue;
@@ -1534,13 +1639,27 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			for (k = 0; k < rp->target->nstrings; ++k)
 			{
 				string_ty	*target_pattern;
+				int		ok;
+
+				/*
+				 * new matcher per recipe, since each one can
+				 * have a different matcher
+				 */
+				trace(("mark\n"));
+				mp = match_new_by_recipe(rp);
 
 				target_pattern = rp->target->string[k];
-				mp = match(target_pattern, target, &rp->pos);
-				if (mp == MATCH_ERROR)
+				ok = match_attempt(mp, target_pattern, target, &rp->pos);
+				if (ok < 0)
+				{
+					match_delete(mp);
 					goto bomb_with_error;
-				if (!mp)
+				}
+				if (!ok)
+				{
+					match_delete(mp);
 					continue;
+				}
 
 				status2 =
 					graph_check_recipe
@@ -1552,16 +1671,15 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 						implicit_allowed,
 						mp
 					);
+				mp = 0;
 
 				switch (status2)
 				{
 				case graph_build_status_error:
-					match_free(mp);
 					trace(("child build error\n"));
 					goto bomb_with_error;
 
 				case graph_build_status_backtrack:
-					match_free(mp);
 					trace(("child build backtrack\n"));
 					gp->statistic.implicit_not_applicable++;
 					continue;
@@ -1833,7 +1951,7 @@ graph_build_file(gp, target, preference, result, implicit_allowed)
 			(
 				leaf == leaf_ness_leaf_explicit
 			&&
-				cook_mtime_oldest(target, (long *)0) == 0
+				cook_mtime_oldest(target, (long *)0, 32767L) == 0
 			)
 			{
 				/*
