@@ -59,10 +59,12 @@
  *	opcode_status_ty - indicating success or failure
  */
 
-static opcode_status_ty run_it _((opcode_list_ty *, const match_ty *));
+static opcode_status_ty run_it _((graph_ty *gp, opcode_list_ty *,
+	const match_ty *));
 
 static opcode_status_ty
-run_it(olp, mp)
+run_it(gp, olp, mp)
+	graph_ty	*gp;
 	opcode_list_ty	*olp;
 	const match_ty	*mp;
 {
@@ -70,6 +72,7 @@ run_it(olp, mp)
 	opcode_status_ty status;
 
 	ocp = opcode_context_new(olp, mp);
+	ocp->gp = gp;
 	status = opcode_context_execute_nowait(ocp);
 	opcode_context_delete(ocp);
 	return status;
@@ -306,7 +309,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 		if (option_test(OPTION_FINGERPRINT))
 		{
 			depth2 = 32767;
-			age2 = cook_mtime_oldest(gfp2->filename, &depth2);
+			age2 = cook_mtime_oldest(gfp2->filename, &depth2, depth2);
 			if (age2 < 0)
 			{
 				/* error message already printed */
@@ -317,7 +320,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 		}
 		
 		depth2 = 32767;
-		age2 = cook_mtime_newest(gfp2->filename, &depth2);
+		age2 = cook_mtime_newest(gfp2->filename, &depth2, depth2);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -388,7 +391,7 @@ i18n("\"$filename\" is out of date because the \"forced\" flag is set (reason)")
 
 		gfp2 = grp->input->file[j];
 		depth2 = 32767;
-		age2 = cook_mtime_oldest(gfp2->filename, &depth2);
+		age2 = cook_mtime_oldest(gfp2->filename, &depth2, target_depth+1);
 		if (age2 < 0)
 		{
 			/* error message already printed */
@@ -655,6 +658,7 @@ now younger (reason)")
 						grp->rp->out_of_date,
 						grp->mp
 					);
+				grp->ocp->gp = gp;
 				hostname =
 					host_binding_round_robin
 					(
@@ -793,7 +797,7 @@ now younger (reason)")
 			 * about making it parallel.
 			 */
 			trace(("perform ``use'' clause\n"));
-			result = run_it(grp->rp->up_to_date, grp->mp);
+			result = run_it(gp, grp->rp->up_to_date, grp->mp);
 			if (result != opcode_status_success)
 				status = graph_walk_status_error;
 		}
@@ -848,7 +852,8 @@ ret:
 					 */
 					continue;
 				}
-				t = cook_mtime_oldest(gfp->filename, &depth4);
+				depth4 = 32767;
+				t = cook_mtime_oldest(gfp->filename, &depth4, depth4);
 				if (t < 0)
 				{
 					status = graph_walk_status_error;
@@ -948,7 +953,8 @@ ret:
 			long		depth2;
 	
 			gfp2 = grp->input->file[j];
-			age2 = cook_mtime_newest(gfp2->filename, &depth2);
+			depth2 = 32767;
+			age2 = cook_mtime_newest(gfp2->filename, &depth2, depth2);
 			if (age2 > need_age_youngest)
 				need_age_youngest = age2;
 		}
@@ -970,7 +976,8 @@ ret:
 			long		depth3;
 
 			gfp = grp->output->file[j];
-			age3 = cook_mtime_newest(gfp->filename, &depth3);
+			depth3 = 32767;
+			age3 = cook_mtime_newest(gfp->filename, &depth3, depth3);
 			if (age3 > 0 && depth3 == 0 && age3 < need_age_youngest)
 				os_mtime_adjust(gfp->filename, need_age_youngest);
 		}
