@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,19 @@
  * MANIFEST: functions to perform a post-order traversal of a dependency graph
  */
 
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/stddef.h>
 #include <ac/stdio.h>
 #include <ac/stdlib.h>
+#include <ac/time.h>
+#ifdef HAVE_WAIT3
+#include <sys/resource.h>
+#endif
 #include <sys/wait.h>
 
 #include <desist.h>
 #include <error_intl.h>
+#include <fingerprint/sync.h>
 #include <graph.h>
 #include <graph/check.h>
 #include <graph/file.h>
@@ -41,6 +46,7 @@
 #include <id.h>
 #include <id/variable.h>
 #include <itab.h>
+#include <opcode/context.h>
 #include <option.h>
 #include <recipe.h> /* for tracing */
 #include <star.h>
@@ -504,6 +510,7 @@ graph_walk_inner(gp, func, nproc)
 		{
 			if (desist_requested())
 				goto desist;
+			fp_sync();
 
 			/*
 			 * Extract a recipe from the list.  Order does
@@ -651,6 +658,51 @@ graph_walk_inner(gp, func, nproc)
 					status = graph_walk_status_done;
 				break;
 			}
+
+#ifdef HAVE_WAIT3
+			/*
+			 * We want to see if any children have exited.
+			 * Do not block (that's why wait3 is used).
+			 * Only do one at a time.
+			 */
+			if (itp->load > 0)
+			{
+				int		pid;
+				int		exit_status;
+				struct rusage	ru;
+	
+				trace(("mark\n"));
+				pid = wait3(&exit_status, WNOHANG, &ru);
+				trace(("pid = %d\n", pid));
+				if (pid < 0)
+				{
+					sub_context_ty	*scp;
+	
+					if (errno == EINTR)
+						continue;
+					scp = sub_context_new();
+					sub_errno_set(scp);
+					fatal_intl(scp, i18n("wait(): $errno"));
+					/* NOTREACHED */
+				}
+				else if (pid > 0)
+				{
+					trace(("es = 0x%04X\n", exit_status));
+					grp = itab_query(itp, pid);
+					if (grp)
+					{
+						/* if it's one of ours... */
+						trace(("...waited\n"));
+						assert(pid == graph_recipe_getpid(grp));
+						if (grp->ocp->rup)
+							*(grp->ocp->rup) = ru;
+						graph_recipe_waited(grp, exit_status);
+						itab_delete(itp, pid);
+						goto run_a_recipe;
+					}
+				}
+			}
+#endif /* HAVE_WAIT3 */
 		}
 
 		/*
@@ -661,9 +713,16 @@ graph_walk_inner(gp, func, nproc)
 		{
 			int		pid;
 			int		exit_status;
+#ifdef HAVE_WAIT3
+			struct rusage	ru;
+#endif
 
 			trace(("mark\n"));
+#ifdef HAVE_WAIT3
+			pid = wait3(&exit_status, 0, &ru);
+#else
 			pid = wait(&exit_status);
+#endif
 			trace(("pid = %d\n", pid));
 			assert(pid != 0);
 			if (pid == 0)
@@ -686,6 +745,10 @@ graph_walk_inner(gp, func, nproc)
 				/* if it's one of ours... */
 				trace(("...waited\n"));
 				assert(pid == graph_recipe_getpid(grp));
+#ifdef HAVE_WAIT3
+				if (grp->ocp->rup)
+					*(grp->ocp->rup) = ru;
+#endif
 				graph_recipe_waited(grp, exit_status);
 				itab_delete(itp, pid);
 				goto run_a_recipe;
