@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -311,3 +311,16 @@ id_interpret(idp, ocp, pp)
 	assert(idp->method->interpret);
 	return idp->method->interpret(idp, ocp, pp);
 }
+
+
+int
+id_interpret_script(idp, ocp, pp)
+	id_ty		*idp;
+	struct opcode_context_ty *ocp;
+	const struct expr_position_ty *pp;
+{
+	assert(idp);
+	assert(idp->method);
+	assert(idp->method->script);
+	return idp->method->script(idp, ocp, pp);
+}
