@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -30,13 +30,13 @@
 #include <ac/string.h>
 #include <ac/stdlib.h>
 #include <ac/stdarg.h>
-#include <ac/stdio.h>
 
 #include <error.h>
 #include <error_intl.h>
 #include <expr.h>
 #include <expr/list.h> /* for parse.gen.h */
 #include <hashline.h>
+#include <input/file_text.h>
 #include <lex.h>
 #include <lex/filename.h>
 #include <lex/filenamelist.h>
@@ -47,6 +47,7 @@
 #include <symtab.h>
 #include <star.h>
 #include <str_list.h>
+#include <stracc.h>
 #include <trace.h>
 #include <parse.gen.h>	/* must be last */
 #include <hashline.gen.h> /* must be last */
@@ -82,7 +83,7 @@ typedef struct lex_ty lex_ty;
 struct lex_ty
 {
 	lex_filename_ty	filename;	/* name of file being analysed	*/
-	FILE		*l_file;	/* open file structure of the file */
+	input_ty	*l_file;	/* open file structure of the file */
 	long		l_line;		/* the line number we are up to	*/
 	meta_ty		l_mback;	/* backup for meta chars	*/
 	lex_ty		*l_chain;	/* file this one is an insert from */
@@ -91,15 +92,6 @@ struct lex_ty
 	int		have_said_eof_token;
 };
 
-typedef struct stracc stracc;
-struct	stracc
-{
-	size_t		sa_max;		/* size of string accum buffer	*/
-	size_t		sa_len;		/* number of chars accumulated	*/
-	char		*sa_buf;	/* the string accum buffer	*/
-	int		sa_inuse;
-};
-
 static	lex_ty		*root;		/* root of insert list		*/
 static	stracc		saroot;		/* root of string accum list	*/
 static	int		errcnt;		/* count of errors to date	*/
@@ -139,8 +131,10 @@ lex_initialize()
 
 	static keyword_ty parse_keyword[] =
 	{
+		{ "+=",			PLUS_EQUALS,		},
 		{ ":",			COLON,			},
 		{ "::",			COLON2,			},
+		{ ";",			SEMICOLON,		},
 		{ "=",			EQUALS,			},
 		{ "[",			LBRAK,			},
 		{ "]",			RBRAK,			},
@@ -151,15 +145,14 @@ lex_initialize()
 		{ "host-binding",	HOST_BINDING,		},
 		{ "if",			IF,			},
 		{ "loop",		LOOP,			},
+		{ "loopstop",		LOOPSTOP,		},
 		{ "return",		RETURN,			},
 		{ "set",		SET,			},
 		{ "single-thread",	SINGLE_THREAD,		},
 		{ "then",		THEN,			},
-		{ "{"/*}*/,		LBRACE,			},
-		{ /*{*/"}",		RBRACE,			},
-		{ ";",			SEMICOLON,		},
-		{ "loopstop",		LOOPSTOP,		},
 		{ "unsetenv",		UNSETENV,		},
+		{ "{",			LBRACE,			},
+		{ "}",			RBRACE,			},
 	};
 	static keyword_ty hash_directive[] =
 	{
@@ -264,17 +257,7 @@ lex_open(logical, physical)
 	}
 	new = mem_alloc_clear(sizeof(lex_ty));
 	lex_filename_constructor(&new->filename, logical, physical);
-	new->l_file = fopen(physical->str_text, "r");
-	if (!new->l_file)
-	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", physical);
-		fatal_intl(scp, i18n("open $filename: $errno"));
-		/* NOTREACHED */
-	}
+	new->l_file = input_file_text_open(physical);
 	new->l_line = 1;
 	new->l_bol = 1;
 	new->l_chain = root;
@@ -321,7 +304,7 @@ lex_close()
 	trace(("lex_close()\n{\n"/*}*/));
 	assert(root);
 	old = root;
-	fclose(root->l_file);
+	input_delete(root->l_file);
 	root = old->l_chain;
 	if (!root && errcnt)
 	{
@@ -413,25 +396,9 @@ byte()
 		/*
 		 * fetch the next input character
 		 */
-		c = fgetc(root->l_file);
+		c = input_getc(root->l_file);
 		if (c != EOF)
 			break;
-		if (ferror(root->l_file))
-		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set
-			(
-				scp,
-				"File_Name",
-				"%S",
-				root->filename.physical
-			);
-			fatal_intl(scp, i18n("read $filename: $errno"));
-			/* NOTREACHED */
-		}
 		if (!root->l_chain || !root->have_said_eof_token)
 			break;
 		lex_close();
@@ -494,164 +461,16 @@ byte_undo(c)
 
 	case '\n':
 		root->l_line--;
+		/* fall through... */
+
 	default:
-		ungetc(c, root->l_file);
+		input_ungetc(root->l_file, c);
 		break;
 	}
 	trace((/*{*/"}\n"));
 }
 
 
-/*
- * NAME
- *	sa_open - start accumulating a string.
- *
- * SYNOPSIS
- *	void sa_open(void);
- *
- * DESCRIPTION
- *	Sa_open begins accumulating a string for the lexical analyser.
- *	This allows virtually infinite length constructs within the
- *	lexical analyser.
- */
-
-static void sa_open _((void));
-
-static void
-sa_open()
-{
-	trace(("sa_open()\n{\n"/*}*/));
-	assert(!saroot.sa_inuse);
-	saroot.sa_inuse = 1;
-	saroot.sa_len = 0;
-	trace((/*{*/"}\n"));
-}
-
-
-/*
- * NAME
- *	sa_char - add a character to the accumulating string
- *
- * SYNOPSIS
- *	void sa_char(int c);
- *
- * DESCRIPTION
- *	Sa_char adds a character to the accumulating string.
- *
- * CAVEAT
- *	Sa_open must have been called previously.
- */
-
-static void sa_char _((int));
-
-static void
-sa_char(c)
-	int		c;
-{
-	trace(("sa_char(c = '%c')\n{\n"/*}*/, c));
-	assert(saroot.sa_inuse);
-	if (saroot.sa_len >= saroot.sa_max)
-	{
-		saroot.sa_max += 128;
-		saroot.sa_buf = mem_change_size(saroot.sa_buf, saroot.sa_max);
-	}
-	saroot.sa_buf[saroot.sa_len++] = c;
-	trace((/*{*/"}\n"));
-}
-
-
-/*
- * NAME
- *	sa_close - finish accumulating a string
- *
- * SYNOPSIS
- *	string_ty *sa_close(void);
- *
- * DESCRIPTION
- *	Sa_close finished accumulating a string and
- *	returns a pointer to the string in dynamic memory.
- *
- * CAVEAT
- *	Sa_open must have been called previously.
- *
- *	The value returned by this function is allocated in dynamic memory.
- *	It is the responsibility of the caller to ensure that it is freed when
- *	finished with, by a call to str_free().
- */
-
-static string_ty *sa_close _((void));
-
-static string_ty *
-sa_close()
-{
-	string_ty	*val;
-
-	trace(("sa_close()\n{\n"/*}*/));
-	assert(saroot.sa_inuse);
-	val = str_n_from_c(saroot.sa_buf, saroot.sa_len);
-	saroot.sa_inuse = 0;
-	trace(("return %08lX;\n", val));
-	trace((/*{*/"}\n"));
-	return val;
-}
-
-
-/*
- * NAME
- *	sa_mark - mark place when accumulating a string
- *
- * SYNOPSIS
- *	size_t sa_mark(void);
- *
- * DESACRIPTION
- *	Sa_mark returns an indicator as to where the string being accumulated
- *	is up to.
- *
- * CAVEAT
- *	Sa_open must have been called previously.
- *	Do not use the returned value for anything other than passing to
- *	sa_goto as an argument.
- */
-
-static size_t sa_mark _((void));
-
-static size_t
-sa_mark()
-{
-	assert(saroot.sa_inuse);
-	return (saroot.sa_len);
-}
-
-
-/*
- * NAME
- *	sa_goto - shorten the accumulated string
- *
- * SYNOPSIS
- *	void sa_goto(size_t mark);
- *
- * DESACRIPTION
- *	Sa_goto shotens the accumulating string to a point
- *	previously determined by sa_mark.
- *
- * CAVEAT
- *	Sa_open must have been called previously.
- *	The mark argument must have previously been returned by sa_mark,
- *	and sa_close not called in the interim.
- */
-
-static void sa_goto _((size_t));
-
-static void
-sa_goto(n)
-	size_t		n;
-{
-	assert(saroot.sa_inuse);
-	assert(n <= saroot.sa_len);
-	saroot.sa_len = n;
-}
-
-
 /*
  * NAME
  *	meta_repn - mata_ty representation
@@ -782,6 +601,34 @@ meta(val)
 			val->m_flag = SINGLE;
 			break;
 	
+		case '+':
+			/*
+			 * processing for += token
+			 */
+			switch (mode)
+			{
+				int	c;
+
+			default:
+				c = byte();
+				byte_undo(c);
+				if (c == '=')
+				{
+					val->m_type = NORMAL;
+					val->m_flag = SINGLE;
+					break;
+				}
+				/* fall through... */
+	
+			case LM_DATA:
+			case LM_SQUOTE:
+			case LM_DQUOTE:
+				val->m_type = NORMAL;
+				val->m_flag = BEFORE | AFTER;
+				break;
+			}
+			break;
+
 		case '=':
 		case ';':
 		case ':':
@@ -1309,10 +1156,30 @@ tokenize()
 				meta(&c);
 				if (c.m_char == ':' && c.m_type == NORMAL)
 				{
-					sa_open();
-					sa_char(':');
-					sa_char(':');
-					parse_lval.lv_word = sa_close();
+					sa_open(&saroot);
+					sa_char(&saroot, ':');
+					sa_char(&saroot, ':');
+					parse_lval.lv_word = sa_close(&saroot);
+					state = 1;
+					tok = UNQUOTED_WORD;
+					last = c;
+					goto ret;
+				}
+				meta_undo(&c);
+				c = hold;
+			}
+			if (c.m_char == '+')
+			{
+				meta_ty	hold;
+
+				hold = c;
+				meta(&c);
+				if (c.m_char == '=' && c.m_type == NORMAL)
+				{
+					sa_open(&saroot);
+					sa_char(&saroot, '+');
+					sa_char(&saroot, '=');
+					parse_lval.lv_word = sa_close(&saroot);
 					state = 1;
 					tok = UNQUOTED_WORD;
 					last = c;
@@ -1328,9 +1195,9 @@ tokenize()
 				int	 sloshed;
 
 				sloshed = 0;
-				sa_open();
+				sa_open(&saroot);
 				if (state == 0)
-					mark = sa_mark();
+					mark = sa_mark(&saroot);
 				for (;;)
 				{
 					trace
@@ -1341,7 +1208,7 @@ tokenize()
 						state,
 						mode_repn(mode)
 					));
-					sa_char(c.m_char);
+					sa_char(&saroot, c.m_char);
 					if (c.m_type == SLOSHED)
 					{
 						sloshed = 1;
@@ -1378,8 +1245,8 @@ tokenize()
 							{
 							case 9:
 							case 10:
-								sa_goto(mark);
-								parse_lval.lv_word = sa_close();
+								sa_goto(&saroot, mark);
+								parse_lval.lv_word = sa_close(&saroot);
 								state = 0;
 								last.m_flag = 0; /* don't CAT after this */
 								dataended = DATAEND;
@@ -1398,7 +1265,7 @@ tokenize()
 
 							default:
 								state = 0;
-								mark = sa_mark();
+								mark = sa_mark(&saroot);
 								break;
 							}
 							break;
@@ -1409,7 +1276,7 @@ tokenize()
 								static int	oldmode = LM_SQUOTE;
 
 								oldmode = lex_mode(oldmode);
-								sa_goto(sa_mark()-1);
+								sa_goto(&saroot, sa_mark(&saroot)-1);
 								state = ((state == 11) ? 1 : 11);
 								sloshed = 1;
 							}
@@ -1421,7 +1288,7 @@ tokenize()
 								static int	oldmode = LM_DQUOTE;
 
 								oldmode = lex_mode(oldmode);
-								sa_goto(sa_mark()-1);
+								sa_goto(&saroot, sa_mark(&saroot)-1);
 								state = ((state == 12) ? 1 : 12);
 								sloshed = 1;
 							}
@@ -1553,7 +1420,7 @@ tokenize()
 						break;
 					}
 				}
-				parse_lval.lv_word = sa_close();
+				parse_lval.lv_word = sa_close(&saroot);
 				if (sloshed)
 					tok = WORD;
 				else
