@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -251,19 +251,7 @@ log_open()
 			 * list only to a file
 			 */
 			if (!freopen(option.o_logfile->str_text, "w", stdout))
-			{
-				scp = sub_context_new();
-				sub_errno_set(scp);
-				sub_var_set
-				(
-					scp,
-					"File_Name",
-					"%S",
-					option.o_logfile
-				);
-				fatal_intl(scp, i18n("open $filename: $errno"));
-				/* NOTREACHED */
-			}
+				fatal_intl_open(option.o_logfile->str_text);
 		}
 		/*
 		 * make sterr go to the same place as stdout
@@ -325,11 +313,7 @@ log_open()
 				!freopen(dev_null, "w", stderr)
 			)
 			{
-				scp = sub_context_new();
-				sub_errno_set(scp);
-				sub_var_set(scp, "File_Name", "%s", dev_null);
-				fatal_intl(scp, i18n("open $filename: $errno"));
-				/* NOTREACHED */
+				fatal_intl_open(dev_null);
 			}
 		}
 	}
