@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <builtin.h>
 #include <cook.h>
 #include <error_intl.h>
+#include <fingerprint.h>
 #include <help.h>
 #include <id.h>
 #include <id/variable.h>
@@ -57,11 +58,14 @@ enum
 	arglex_token_errok_not,
 	arglex_token_fingerprint,
 	arglex_token_fingerprint_not,
+	arglex_token_fingerprint_update,
 	arglex_token_force,
 	arglex_token_force_not,
 	arglex_token_include,
 	arglex_token_include_cooked,
 	arglex_token_include_cooked_not,
+	arglex_token_include_cooked_warning,
+	arglex_token_include_cooked_warning_not,
 	arglex_token_log,
 	arglex_token_log_not,
 	arglex_token_metering,
@@ -91,7 +95,8 @@ enum
 	arglex_token_tty,
 	arglex_token_tty_not,
 	arglex_token_update,
-	arglex_token_update_not
+	arglex_token_update_not,
+	arglex_token_web
 
 	/*
 	 * When you add an option to this list, you must
@@ -115,13 +120,17 @@ static arglex_table_ty argtab[] =
 	{ "-No_Errok",		(arglex_token_ty)arglex_token_errok_not,	},
 	{ "-FingerPrint",	(arglex_token_ty)arglex_token_fingerprint, },
 	{ "-No_FingerPrint",	(arglex_token_ty)arglex_token_fingerprint_not, },
+	{ "-FingerPrint_Update", (arglex_token_ty)arglex_token_fingerprint_update, },
 	{ "-Forced",		(arglex_token_ty)arglex_token_force,	},
 	{ "-No_Forced",		(arglex_token_ty)arglex_token_force_not,	},
+	{ "-HyperText_Markup_Language", (arglex_token_ty)arglex_token_web, },
 	{ "-Include",		(arglex_token_ty)arglex_token_include,	},
 	{ "-\\I*",		(arglex_token_ty)arglex_token_include,	},
 	{ "-Include_Cooked",	(arglex_token_ty)arglex_token_include_cooked, },
+	{ "-Include_Cooked_Warning", (arglex_token_ty)arglex_token_include_cooked_warning, },
 	{ "-Jobs",		(arglex_token_ty)arglex_token_parallel, },
 	{ "-No_Include_Cooked",	(arglex_token_ty)arglex_token_include_cooked_not, },
+	{ "-No_Include_Cooked_Warning", (arglex_token_ty)arglex_token_include_cooked_warning_not, },
 	{ "-LOg",		(arglex_token_ty)arglex_token_log,	},
 	{ "-List",		(arglex_token_ty)arglex_token_log,	},
 	{ "-No_LOg",		(arglex_token_ty)arglex_token_log_not,	},
@@ -151,7 +160,10 @@ static arglex_table_ty argtab[] =
 	{ "-TRace",		(arglex_token_ty)arglex_token_reason,	},
 	{ "-No_TRace",		(arglex_token_ty)arglex_token_reason_not,	},
 	{ "-Update",		(arglex_token_ty)arglex_token_update,	},
+	{ "-Time_Adjust",	(arglex_token_ty)arglex_token_update,	},
 	{ "-No_Update",		(arglex_token_ty)arglex_token_update_not, },
+	{ "-No_Time_Adjust",	(arglex_token_ty)arglex_token_update_not, },
+	{ "-Web",		(arglex_token_ty)arglex_token_web,	},
 	{ 0, (arglex_token_ty)0, }, /* end marker */
 };
 
@@ -210,8 +222,10 @@ argparse(level)
 	option_number_ty type;
 	string_ty	*s;
 	sub_context_ty	*scp;
+	int		fingerprint_update;
 
 	type = -1;
+	fingerprint_update = 0;
 	switch (arglex())
 	{
 	case arglex_token_help:
@@ -250,9 +264,7 @@ argparse(level)
 		case arglex_token_include:
 			if (arglex() != arglex_token_string)
 			{
-				scp = sub_context_new();
-				sub_var_set(scp, "Name", "%s", arglex_token_name(arglex_token_include));
-				fatal_intl(scp, i18n("$name needs string"));
+				arg_needs_string(arglex_token_include, usage);
 				/* NOTREACHED */
 			}
 			s = str_from_c(arglex_value.alv_string);
@@ -266,19 +278,7 @@ argparse(level)
 			if (option_already(type, level))
 			{
 				too_many:
-				scp = sub_context_new();
-				sub_var_set
-				(
-					scp,
-					"Name",
-					"%s",
-					arglex_value.alv_string
-				);
-				fatal_intl
-				(
-					scp,
-					i18n("duplicate \"$name\" option")
-				);
+				arg_duplicate_cur(usage);
 				/* NOTREACHED */
 			}
 			option_set(type, level, 1);
@@ -362,6 +362,14 @@ argparse(level)
 			type = OPTION_INCLUDE_COOKED;
 			goto normal_off;
 
+		case arglex_token_include_cooked_warning:
+			type = OPTION_INCLUDE_COOKED_WARNING;
+			goto normal_on;
+
+		case arglex_token_include_cooked_warning_not:
+			type = OPTION_INCLUDE_COOKED_WARNING;
+			goto normal_off;
+
 		case arglex_token_silent:
 			type = OPTION_SILENT;
 			goto normal_on;
@@ -426,6 +434,14 @@ argparse(level)
 			type = OPTION_FINGERPRINT;
 			goto normal_off;
 
+		case arglex_token_fingerprint_update:
+			if (level != OPTION_LEVEL_COMMAND_LINE)
+				goto not_in_env;
+			if (option.fingerprint_update)
+				goto too_many;
+			option.fingerprint_update++;
+			break;
+
 		case arglex_token_pairs:
 			if (level != OPTION_LEVEL_COMMAND_LINE)
 				goto not_in_env;
@@ -442,6 +458,14 @@ argparse(level)
 			option.script++;
 			break;
 
+		case arglex_token_web:
+			if (level != OPTION_LEVEL_COMMAND_LINE)
+				goto not_in_env;
+			if (option.web)
+				goto too_many;
+			option.web++;
+			break;
+
 		case arglex_token_string:
 			if (level != OPTION_LEVEL_COMMAND_LINE)
 			{
@@ -585,7 +609,7 @@ main(argc, argv)
 	lex_initialize();
 
 	/*
-	 * parse the command line
+	 * parse the COOK environment variable
 	 */
 	arglex_init_from_env(argv[0], argtab);
 	argparse(OPTION_LEVEL_ENVIRONMENT);
@@ -606,6 +630,16 @@ main(argc, argv)
 	if (option_test(OPTION_STAR))
 		star_enable();
 
+	/*
+	 * If we were asked to update the fingerprints, do it here.
+	 * We don't actually ant to read in the cookbook.
+	 */
+	if (option.fingerprint_update)
+	{
+		fp_tweak();
+		quit(0);
+	}
+
 	/*
 	 * read in the cook book
 	 *
@@ -674,6 +708,8 @@ main(argc, argv)
 		retval = cook_pairs(&option.o_target);
 	else if (option.script)
 		retval = cook_script(&option.o_target);
+	else if (option.web)
+		retval = cook_web(&option.o_target);
 	else
 		retval = cook(&option.o_target);
 
