@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -18,14 +18,22 @@
  *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to manipulate command opcodes
+ *
+ * See also: cook/os.c::execute()
  */
 
-#include <errno.h>
+#include <ac/errno.h>
+#include <ac/fcntl.h>
 #include <ac/stdio.h>
 #include <ac/stdlib.h>
 #include <ac/string.h>
 #include <ac/time.h>
 #include <ac/unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>	/* for chmod */
+#ifdef HAVE_WAIT3
+#include <sys/resource.h>
+#endif
 
 #include <error.h>
 #include <error_intl.h>
@@ -130,6 +138,61 @@ id_var_search(key)
 }
 
 
+static string_ty *temporary_filename _((void));
+
+static string_ty *
+temporary_filename()
+{
+#ifdef __CYGWIN32__
+	/*
+	 * For some reason CygWin32's tmpnam() always produces
+	 * filenames which give ``No such file or directory''
+	 */
+	char		iname[100];
+	static int	nnn;
+
+	sprintf(iname, "t%dp%d.tmp", ++nnn, getpid());
+#else
+	char		iname[L_tmpnam];
+
+	tmpnam(iname);
+#endif
+	return str_from_c(iname);
+}
+
+
+static string_ty *dot_temporary_filename _((void));
+
+static string_ty *
+dot_temporary_filename()
+{
+	static long temp_file_number;
+
+	return str_format(".%d.%ld", getpid(), ++temp_file_number);
+}
+
+
+static void chmod_and_check _((const char *, int));
+
+static void
+chmod_and_check(fn, mode)
+	const char	*fn;
+	int		mode;
+{
+	if (chmod(fn, mode))
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%s", fn);
+		sub_var_set(scp, "Number", "0%o", mode);
+		fatal_intl(scp, i18n("chmod $filename $mode: $errno"));
+		/* NOTREACHED */
+	}
+}
+
+
 /*
  * NAME
  *	spawn - execute a command
@@ -158,8 +221,8 @@ spawn(args, input, pid_p, host_binding)
 	string_ty	*host_binding;
 {
 	int		j;
-	FILE		*fp;
-	char		iname[L_tmpnam];
+	int		fd;
+	string_ty	*iname;
 	int		pid;
 	opcode_status_ty status;
 	static char	**argv;
@@ -170,49 +233,36 @@ spawn(args, input, pid_p, host_binding)
 
 	trace(("execute()\n{\n"/*}*/));
 	assert(args);
-	assert(args->nstrings > 0);
 
 	/*
 	 * build the input file, if required
 	 */
 	status = opcode_status_error;
-	fp = 0;
+	fd = -1;
+	iname = 0;
 	if (input)
 	{
 		/*
 		 * He has given a string to be used as input to the command,
 		 * so write it out to a file, and then redirect the input.
 		 */
-		tmpnam(iname);
-		fp = fopen(iname, "w+");
-		if (!fp)
+		iname = temporary_filename();
+		fd = open(iname->str_text, O_RDWR | O_CREAT | O_TRUNC, 0666);
+		if (fd < 0)
+			fatal_intl_open(iname->str_text);
+		if (unlink(iname->str_text))
 		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("open $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_unlink(iname->str_text);
 			goto done;
 		}
-		if (unlink(iname))
+		if
+		(
+			write(fd, input->str_text, input->str_length) < 0
+		||
+			lseek(fd, 0L, SEEK_SET)
+		)
 		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("unlink $filename: $errno"));
-			sub_context_delete(scp);
-			fclose(fp);
-			goto done;
-		}
-		fputs(input->str_text, fp);
-		if (ferror(fp) || fseek(fp, 0L, 0))
-		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("write $filename: $errno"));
-			sub_context_delete(scp);
-			fclose(fp);
+			error_intl_write(iname->str_text);
 			goto done;
 		}
 	}
@@ -227,9 +277,11 @@ spawn(args, input, pid_p, host_binding)
 		string_ty	*s;
 		string_ty	*rcmd;
 		string_ty	*rcmdq;
-		string_ty	*temp_file_name;
-		static long	temp_file_number;
+		string_ty	*result_fn;
+		FILE		*result_fp;
 		string_ty	*cwd;
+		string_ty	*script_fn;
+		FILE		*script_fp;
 
 		/*
 		 * Work out the name of the remote shell command.
@@ -244,7 +296,7 @@ spawn(args, input, pid_p, host_binding)
 			static string_ty *rsh;
 
 			if (!rsh)
-				rsh = str_from_c("rsh");
+				rsh = str_from_c(CONF_REMOTE_SHELL);
 			string_list_constructor(&cmd);
 			string_list_append(&cmd, rsh);
 		}
@@ -258,18 +310,17 @@ spawn(args, input, pid_p, host_binding)
 		 * The remote end will need to change directory to where
 		 * we are now, since it is *highly* unlikely that we are
 		 * in our home directory.
-		 *
-		 * The comamnd itself will need to be quoted, because
-		 * the command is passed to a shell th athe other end,
-		 * twice!
-		 *
-		 * Because rsh(1) always returns an exit status of zero,
-		 * we have to make other arrangements to obtain the exit
-		 * status.
 		 */
-		rcmd = wl2str(args, 0, args->nstrings, (char *)0);
-		rcmdq = quote(rcmd);
-		str_free(rcmd);
+		cwd = os_curdir(); /* do not str_free this */
+
+		/*
+		 * We are going to create a small shell script contining
+		 * the command to be executed.  This deals with various
+		 * weird and obscure $SHELL settings, and it also deals
+		 * with command lines so long that rsh barfs.
+		 */
+		script_fn = dot_temporary_filename();
+		script_fp = fopen_and_check(script_fn->str_text, "w");
 
 		/*
 		 * what shell are we using
@@ -277,30 +328,58 @@ spawn(args, input, pid_p, host_binding)
 		shell = getenv("SHELL");
 		if (!shell || !*shell)
 			shell = CONF_SHELL;
+		fprintf(script_fp, "#!%s\n", shell);
 
 		/*
+		 * Now write the actual command to the script file.
+		 */
+		rcmd = wl2str(args, 0, args->nstrings, (char *)0);
+		fprintf(script_fp, "%s\n", rcmd->str_text);
+		str_free(rcmd);
+
+		fflush_and_check(script_fp, script_fn->str_text);
+		fclose_and_check(script_fp, script_fn->str_text);
+		chmod_and_check(script_fn->str_text, 0755);
+
+		/*
+		 * This is the result file.  It is where the remote
+		 * command will stash its exit status.	We seed it with
+		 * failure in case the rsh fails altogether.
+		 */
+		result_fn = dot_temporary_filename();
+		result_fp = fopen_and_check(result_fn->str_text, "w");
+		fprintf(result_fp, "42\n");
+		fflush_and_check(result_fp, result_fn->str_text);
+		fclose_and_check(result_fp, result_fn->str_text);
+
+		/*
+		 * Because rsh(1) always returns an exit status of zero,
+		 * we have to make other arrangements to obtain the exit
+		 * status.
+		 *
 		 * We actually need to be a more devious, so that we can
 		 * get the exit status back.  Write it to a temporary
 		 * file server side, then read and remove it client side.
 		 *
 		 * This is the server side command.  It gets quoted
 		 * again to protect it from the client side shell.
+		 *
+		 * We use sh explicitly, because the shell at the other
+		 * end could be csh, tcsh, or something even stranger,
+		 * and we need to guarantee the command will execute
+		 * exactly the way we need.
 		 */
-		temp_file_name =
-			str_format(".%d.%ld", getpid(), ++temp_file_number);
-		cwd = os_curdir(); /* do not str_free this */
 		rcmd =
 			str_format
 			(
-				"cd %S && ( %s %s %S ) ; echo $? > %S/%S",
+				"sh -c 'cd %S && sh %s %S/%S; echo $? > %S/%S'",
 				cwd,
-				shell,
 				(option_test(OPTION_ERROK) ? "-c" : "-ce"),
-				rcmdq,
 				cwd,
-				temp_file_name
+				script_fn,
+				cwd,
+				result_fn
 			);
-		str_free(rcmdq);
 		rcmdq = quote(rcmd);
 		str_free(rcmd);
 		string_list_append(&cmd, rcmdq);
@@ -308,17 +387,26 @@ spawn(args, input, pid_p, host_binding)
 
 		/*
 		 * This is the rest of the server side command.
+		 *
+		 * On some systems this gives ``Text file busy'' errors,
+		 * as if the script file has yet to be released by
+		 * the kernel.	Since we *aren't* executing it when the
+		 * RM command runs, I have no idea how to avoid this
+		 * (other than ignoring it with -f).  Maybe this is some
+		 * kind of NFS latency?
 		 */
 		s =
 			str_format
 			(
-				"&& exit `cat %S;rm %S`",
-				temp_file_name,
-				temp_file_name
+				"&& exit `cat %S;rm -f %S %S`",
+				result_fn,
+				result_fn,
+				script_fn
 			);
 		string_list_append(&cmd, s);
 		str_free(s);
-		str_free(temp_file_name);
+		str_free(result_fn);
+		str_free(script_fn);
 
 		/*
 		 * Because the command has a semicolon and back quotes,
@@ -332,7 +420,7 @@ spawn(args, input, pid_p, host_binding)
 		 * Build the final command to be executed.
 		 * It even cleans up after itself.
 		 */
-		s = str_from_c(shell);
+		s = str_from_c("sh");
 		string_list_append(&cmd, s);
 		str_free(s);
 		s = str_from_c("-c");
@@ -410,8 +498,6 @@ spawn(args, input, pid_p, host_binding)
 	}
 	if (cmd.nstrings == 0)
 	{
-		if (fp)
-			fclose(fp);
 		string_list_destructor(&cmd);
 		status = opcode_status_success;
 		goto done;
@@ -436,7 +522,7 @@ spawn(args, input, pid_p, host_binding)
 		/*
 		 * child
 		 */
-		if (fp)
+		if (fd >= 0)
 		{
 			if (close(0) && errno != EBADF)
 			{
@@ -454,7 +540,7 @@ spawn(args, input, pid_p, host_binding)
 				sub_context_delete(scp);
 				str_free(fn0);
 			}
-			if (dup(fileno(fp)) < 0)
+			if (dup(fd) < 0)
 			{
 				scp = sub_context_new();
 				sub_errno_set(scp);
@@ -462,6 +548,7 @@ spawn(args, input, pid_p, host_binding)
 				/* NOTREACHED */
 				sub_context_delete(scp);
 			}
+			close(fd);
 		}
 		if (argv[0][0] == '/')
 			execv(argv[0], argv);
@@ -478,8 +565,11 @@ spawn(args, input, pid_p, host_binding)
 		/*
 		 * parent
 		 */
-		if (fp)
-			fclose(fp);
+		if (fd >= 0)
+		{
+			close(fd);
+			fd = -1;
+		}
 		string_list_destructor(&cmd);
 		status = opcode_status_wait;
 		trace(("pid = %d;\n", pid));
@@ -487,6 +577,10 @@ spawn(args, input, pid_p, host_binding)
 		break;
 	}
 	done:
+	if (fd >= 0)
+		close(fd);
+	if (iname)
+		str_free(iname);
 	trace(("return %s;\n", opcode_status_name(status)));
 	trace((/*{*/"}\n"));
 	return status;
@@ -511,7 +605,7 @@ spawn(args, input, pid_p, host_binding)
 static void meter_ptime _((double, char *));
 
 static void
-meter_ptime(t,s,e)
+meter_ptime(t, s)
 	double		t;
 	char		*s;
 {
@@ -543,7 +637,7 @@ meter_ptime(t,s,e)
  *	the metered information.
  */
 
-static void meter_end _((struct rusage *));
+static void meter_print _((struct rusage *));
 
 static void
 meter_print(rup)
@@ -684,8 +778,8 @@ execute(op, icp)
 		 * prepare for metering
 		 */
 #ifdef HAVE_WAIT3
-		if (option_test(OPTION_METERING) && !icp->rup)
-			icp->rup = mem_alloc(struct rusage);
+		if (option_test(OPTION_METER) && !icp->rup)
+			icp->rup = mem_alloc(sizeof(struct rusage));
 #endif
 
 		/*
@@ -703,7 +797,7 @@ execute(op, icp)
 			goto done;
 resume:
 #ifdef HAVE_WAIT3
-		if (option_test(OPTION_METERING))
+		if (option_test(OPTION_METER))
 		{
 			assert(icp->rup);
 			meter_print(icp->rup);
@@ -814,6 +908,13 @@ script(op, icp)
 	flag_set_options(flags, OPTION_LEVEL_EXECUTE);
 	flag_delete(flags);
 
+	/*
+	 * If the command is empty (e.g. [print ...]; or [write ...];)
+	 * do nothing at all.
+	 */
+	if (wlp->nstrings == 0)
+		goto done;
+
 	/*
 	 * echo the command if required
 	 */
