@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1997, 1998 Peter Miller;
+ *	Copyright (C) 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  * MANIFEST: functions to manipulate opcode contexts
  */
 
-#include <errno.h>
+#include <ac/errno.h>
 #include <ac/stddef.h>
 #include <sys/wait.h>
 
@@ -28,7 +28,7 @@
 #include <error_intl.h>
 #include <id.h>
 #include <id/variable.h>
-#include <match.h>
+#include <match/stack.h>
 #include <mem.h>
 #include <opcode.h>
 #include <opcode/context.h>
@@ -77,6 +77,7 @@ opcode_context_new(olp, mp)
 	ocp->pid = 0;
 	ocp->exit_status = 0;
 	ocp->rup = 0;
+	ocp->gp = 0;
 
 	ocp->target = 0;
 	ocp->targets = 0;
