@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -78,15 +78,18 @@ static flag_ty flag;
 
 void
 option_set(o, level, state)
-	option_number_ty	o;
+	option_number_ty o;
 	option_level_ty	level;
 	int		state;
 {
+	trace(("option_set(o = %s, level = %s, state = %d)\n{\n",
+		option_number_name(o), option_level_name(level), state));
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
 	if (state)
 		flag.o_flag[(size_t)o] |= 3L << (2 * (int)level);
 	else
 		flag.o_flag[(size_t)o] |= 1L << (2 * (int)level);
+	trace(("}\n"));
 }
 
 
@@ -107,11 +110,17 @@ option_set(o, level, state)
 
 int
 option_already(o, level)
-	option_number_ty	o;
+	option_number_ty o;
 	option_level_ty	level;
 {
+	int		result;
+
+	trace(("option_already(o = %s, level = %s)\n{\n", option_number_name(o), option_level_name(level)));
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
-	return ((flag.o_flag[(size_t)o] >> (2 * (int)level)) & 1) != 0;
+	result = (((flag.o_flag[(size_t)o] >> (2 * (int)level)) & 1) != 0);
+	trace(("return %d;\n", result));
+	trace(("}\n"));
+	return result;
 }
 
 
@@ -135,8 +144,10 @@ option_undo(o, level)
 	option_number_ty	o;
 	option_level_ty	level;
 {
+	trace(("option_undo(o = %s, level = %s)\n{\n", option_number_name(o), option_level_name(level)));
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
 	flag.o_flag[(size_t)o] &= ~(3L << (2 * (int)level));
+	trace(("}\n"));
 }
 
 
@@ -161,8 +172,10 @@ option_undo_level(level)
 {
 	int		o;
 
+	trace(("option_undo_level(level = %s)\n{\n", option_level_name(level)));
 	for (o = 0; o < (int)OPTION_max; ++o)
 		option_undo((option_number_ty)o, level);
+	trace(("}\n"));
 }
 
 
@@ -187,12 +200,17 @@ option_test(o)
 {
 	unsigned long	*op;
 	unsigned long	mask;
+	int		result;
 
+	trace(("option_test(o = %s)\n{\n", option_number_name(o)));
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
 	op = &flag.o_flag[(size_t)o];
 	mask = *op & 0x55555555;
 	mask &= -mask; /* get LSB */
-	return (*op & (mask << 1)) != 0;
+	result = ((*op & (mask << 1)) != 0);
+	trace(("return %d;\n", result));
+	trace(("}\n"));
+	return result;
 }
 
 
@@ -220,10 +238,23 @@ option_tidy_up()
 	string_ty	*s;
 	string_ty	*s1;
 
+	/*
+	 * Set the defaults before we do anything else,
+	 * the rest of tidy_up depends on them.
+	 */
+	trace(("option_tidy_up()\n{\n"));
+	option_set(OPTION_ACTION, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_CASCADE, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_FINGERPRINT_WRITE, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_INCLUDE_COOKED, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_INCLUDE_COOKED_WARNING, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_LOGGING, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_STRIP_DOT, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_TERMINAL, OPTION_LEVEL_DEFAULT, 1);
+
 	/*
 	 * user's library
 	 */
-	trace(("tidy up\n"));
 	s = os_accdir();
 	assert(s);
 	s1 = str_format("%S/.%s", s, progname_get());
@@ -306,14 +337,7 @@ option_tidy_up()
 		option.o_logfile = str_format("%S.%s", s, sp);
 		str_free(s);
 	}
-
-	option_set(OPTION_ACTION, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_CASCADE, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_LOGGING, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_TERMINAL, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_STRIP_DOT, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_INCLUDE_COOKED, OPTION_LEVEL_DEFAULT, 1);
-	option_set(OPTION_FINGERPRINT_WRITE, OPTION_LEVEL_DEFAULT, 1);
+	trace(("}\n"));
 }
 
 
@@ -335,11 +359,13 @@ option_tidy_up()
 void
 option_set_errors()
 {
+	trace(("option_set_errors()\n{\n"));
 	option_set(OPTION_SILENT, OPTION_LEVEL_ERROR, 0);
 	option_set(OPTION_ACTION, OPTION_LEVEL_ERROR, 0);
 	option_set(OPTION_ERROK, OPTION_LEVEL_ERROR, 0);
 	option_set(OPTION_METER, OPTION_LEVEL_ERROR, 0);
 	option_set(OPTION_PERSEVERE, OPTION_LEVEL_ERROR, 0);
+	trace(("}\n"));
 }
 
 
@@ -364,3 +390,65 @@ option_flag_state_set(p)
 	flag = *fp;
 	mem_free(p);
 }
+
+
+const char *
+option_level_name(lvl)
+	option_level_ty	lvl;
+{
+	switch (lvl)
+	{
+	case OPTION_LEVEL_ERROR: return "OPTION_LEVEL_ERROR";
+	case OPTION_LEVEL_AUTO: return "OPTION_LEVEL_AUTO";
+	case OPTION_LEVEL_COMMAND_LINE: return "OPTION_LEVEL_COMMAND_LINE";
+	case OPTION_LEVEL_EXECUTE: return "OPTION_LEVEL_EXECUTE";
+	case OPTION_LEVEL_RECIPE: return "OPTION_LEVEL_RECIPE";
+	case OPTION_LEVEL_COOKBOOK: return "OPTION_LEVEL_COOKBOOK";
+	case OPTION_LEVEL_ENVIRONMENT: return "OPTION_LEVEL_ENVIRONMENT";
+	case OPTION_LEVEL_DEFAULT: return "OPTION_LEVEL_DEFAULT";
+	}
+	return "option level unknown";
+}
+
+
+const char *
+option_number_name(o)
+	option_number_ty o;
+{
+	switch (o)
+	{
+	case OPTION_ACTION: return "OPTION_ACTION";
+	case OPTION_BOOK: return "OPTION_BOOK";
+	case OPTION_CASCADE: return "OPTION_CASCADE";
+	case OPTION_CMDFILE: return "OPTION_CMDFILE";
+	case OPTION_DISASSEMBLE: return "OPTION_DISASSEMBLE";
+	case OPTION_ERROK: return "OPTION_ERROK";
+	case OPTION_FINGERPRINT: return "OPTION_FINGERPRINT";
+	case OPTION_FINGERPRINT_WRITE: return "OPTION_FINGERPRINT_WRITE";
+	case OPTION_FORCE: return "OPTION_FORCE";
+	case OPTION_GATEFIRST: return "OPTION_GATEFIRST";
+	case OPTION_IMPLICIT_ALLOWED: return "OPTION_IMPLICIT_ALLOWED";
+	case OPTION_INCLUDE_COOKED: return "OPTION_INCLUDE_COOKED";
+	case OPTION_INCLUDE_COOKED_WARNING: return "OPTION_INCLUDE_COOKED_WARNING";
+	case OPTION_INVALIDATE_STAT_CACHE: return "OPTION_INVALIDATE_STAT_CACHE";
+	case OPTION_LOGGING: return "OPTION_LOGGING";
+	case OPTION_METER: return "OPTION_METER";
+	case OPTION_MKDIR: return "OPTION_MKDIR";
+	case OPTION_PERSEVERE: return "OPTION_PERSEVERE";
+	case OPTION_PRECIOUS: return "OPTION_PRECIOUS";
+	case OPTION_REASON: return "OPTION_REASON";
+	case OPTION_RECURSE: return "OPTION_RECURSE";
+	case OPTION_SHALLOW: return "OPTION_SHALLOW";
+	case OPTION_SILENT: return "OPTION_SILENT";
+	case OPTION_STAR: return "OPTION_STAR";
+	case OPTION_STRIP_DOT: return "OPTION_STRIP_DOT";
+	case OPTION_TERMINAL: return "OPTION_TERMINAL";
+	case OPTION_TOUCH: return "OPTION_TOUCH";
+	case OPTION_UNLINK: return "OPTION_UNLINK";
+	case OPTION_UPDATE: return "OPTION_UPDATE";
+	case OPTION_UPDATE_MAX: return "OPTION_UPDATE_MAX";
+	case OPTION_MATCH_MODE_REGEX: return "OPTION_MATCH_MODE_REGEX";
+	case OPTION_max: return "OPTION_max";
+	}
+	return "option number unknown";
+}
