@@ -1,7 +1,7 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997
- *	Peter Miller; All rights reserved.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1999 Peter Miller;
+ *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -56,8 +56,10 @@ enum option_number_ty
 	OPTION_FINGERPRINT,	/* remember file fingerprints */
 	OPTION_FINGERPRINT_WRITE, /* preserve fingerprints if taken */
 	OPTION_FORCE,		/* always execute the commands */
+	OPTION_GATEFIRST,	/* check the gate conditions on a recipe before evaluating the ingredients */
 	OPTION_IMPLICIT_ALLOWED, /* implicit recipes may be used */
 	OPTION_INCLUDE_COOKED,	/* cook the include-cooked include files */
+	OPTION_INCLUDE_COOKED_WARNING,	/* warn of include-cooked problems */
 	OPTION_INVALIDATE_STAT_CACHE,
 	OPTION_LOGGING,
 	OPTION_METER,		/* meter each command */
@@ -75,6 +77,7 @@ enum option_number_ty
 	OPTION_UNLINK,		/* remove targets before running rule body */
 	OPTION_UPDATE,		/* update utime for consistency */
 	OPTION_UPDATE_MAX,	/* update utime for consistency - backwards! */
+	OPTION_MATCH_MODE_REGEX, /* regex pattern matching (as opp native) */
 
 	/*
 	 * If you add to this list, make sure you also add the option to
@@ -94,6 +97,8 @@ struct option_ty
 	string_list_ty	o_vardef;
 	int		pairs;
 	int		script;
+	int		web;
+	int		fingerprint_update;
 };
 
 extern	option_ty	option;
@@ -109,4 +114,7 @@ void option_tidy_up _((void));
 void *option_flag_state_get _((void));
 void option_flag_state_set _((void *));
 
+const char *option_number_name _((option_number_ty));
+const char *option_level_name _((option_level_ty));
+
 #endif /* OPTION_H */
