@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,13 +20,14 @@
  * MANIFEST: functions to isolate operating system interface
  */
 
-#include <ac/stddef.h>
-#include <ac/string.h>
-#include <ac/stdlib.h>
+#include <ac/errno.h>
+#include <ac/fcntl.h>
 #include <ac/limits.h>
-#include <errno.h>
-#include <ac/stdio.h>
 #include <signal.h>
+#include <ac/stddef.h>
+#include <ac/stdio.h>
+#include <ac/stdlib.h>
+#include <ac/string.h>
 #include <ac/time.h>
 #include <ac/unistd.h>
 
@@ -71,14 +72,26 @@ time_t
 os_mtime_oldest(path)
 	string_ty	*path;
 {
-	return stat_cache_oldest(path, 1);
+	time_t		result;
+
+	trace(("os_mtime_oldest(path = \"%s\")\n{\n", path->str_text));
+	result = stat_cache_oldest(path, 1);
+	trace(("return (%ld) %s", (long)result, ctime(&result)));
+	trace(("}\n"));
+	return result;
 }
 
 time_t
 os_mtime_newest(path)
 	string_ty	*path;
 {
-	return stat_cache_newest(path, 1);
+	time_t		result;
+
+	trace(("os_mtime_newest(path = \"%s\")\n{\n", path->str_text));
+	result = stat_cache_newest(path, 1);
+	trace(("return (%ld) %s", (long)result, ctime(&result)));
+	trace(("}\n"));
+	return result;
 }
 
 
@@ -180,7 +193,7 @@ os_mtime_adjust(path, min_age)
 	time_t		mtime;
 	int		err;
 
-	trace(("os_mtime_adjust()\n{\n"));
+	trace(("os_mtime_adjust(path = \"%s\")\n{\n", path->str_text));
 	if (option_test(OPTION_UPDATE) && option_test(OPTION_ACTION))
 	{
 		stat_cache_clear(path);
@@ -410,13 +423,7 @@ os_delete(path, echo)
 	{
 		if (errno != ENOENT)
 		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", path);
-			error_intl(scp, i18n("unlink $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_unlink(path->str_text);
 			option_set_errors();
 			return -1;
 		}
@@ -557,7 +564,8 @@ exit_status(cmd, status, errok)
  * RETURNS
  *	int: zero if the commands succeeds, nonzero if it fails.
  *
- * CAVEAT
+ * SEE ALSO
+ *	cook/opcode/command.c
  */
 
 static int execute _((string_list_ty *, int, int));
@@ -669,6 +677,9 @@ execute(cmd, fd, errok)
  *	If the command succeeds 0 is returned.
  *	If the command fails a diagnostic message is given
  *	and 1 is returned.
+ *
+ * SEE ALSO
+ *	cook/opcode/command.c
  */
 
 int
@@ -678,31 +689,34 @@ os_execute(args, input, errok)
 	int		errok;
 {
 	int		j;
-	FILE		*fp;
+	int		fd;
 	char		iname[L_tmpnam];
 	int		retval;
 
 	assert(args);
 	assert(args->nstrings > 0);
 
-	fp = 0;
+	fd = -1;
 	if (input)
 	{
 		/*
 		 * He has given a string to be used as input to the command,
 		 * so write it out to a file, and then redirect the input.
 		 */
+#ifdef __CYGWIN32__
+		/*
+		 * For some reason CygWin32's tmpnam() always produces
+		 * filenames which give ``No such file or directory''
+		 */
+		{ static int nnn;
+		sprintf(iname, "t%dp%d.tmp", ++nnn, getpid()); }
+#else
 		tmpnam(iname);
-		fp = fopen(iname, "w+");
-		if (!fp)
+#endif
+		fd = open(iname, O_RDWR | O_CREAT | O_TRUNC, 0666);
+		if (fd < 0)
 		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("open $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_open(iname);
 			exec_fails:
 			option_set_errors();
 			retval = -1;
@@ -710,25 +724,17 @@ os_execute(args, input, errok)
 		}
 		if (unlink(iname))
 		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("unlink $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_unlink(iname);
 			goto exec_fails;
 		}
-		fputs(input->str_text, fp);
-		if (ferror(fp) || fseek(fp, 0L, 0))
+		if
+		(
+			write(fd, input->str_text, input->str_length) < 0
+		||
+			lseek(fd, 0L, SEEK_SET)
+		)
 		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", iname);
-			error_intl(scp, i18n("write $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_write(iname);
 			goto exec_fails;
 		}
 	}
@@ -773,21 +779,21 @@ os_execute(args, input, errok)
 					execute
 					(
 						&newcmd,
-						(input ? fileno(fp) : -1),
+						fd,
 						errok
 					);
 				string_list_destructor(&newcmd);
 				if (input)
-					fclose(fp);
+					close(fd);
 				return retval;
 			}
 			++s;
 		}
 	}
-	retval = execute(args, (input ? fileno(fp) : -1), errok);
+	retval = execute(args, fd, errok);
 	if (input)
-		fclose(fp);
-ret:
+		close(fd);
+	ret:
 	return retval;
 }
 
@@ -811,11 +817,17 @@ os_exists(path)
 	string_ty	*path;
 {
 	time_t		mtime;
+	int		result;
 
+	trace(("os_exists(path = \"%s\")\n{\n", path->str_text));
 	mtime = stat_cache_newest(path, 1);
 	if (mtime < 0)
-		return -1;
-	return (mtime != 0);
+		result = -1;
+	else
+		result = (mtime != 0);
+	trace(("return %d\n", result));
+	trace(("}\n"));
+	return result;
 }
 
 
@@ -824,11 +836,37 @@ os_exists_symlink(path)
 	string_ty	*path;
 {
 	time_t		mtime;
+	int		result;
 
+	trace(("os_exists_symlink(path = \"%s\")\n{\n", path->str_text));
 	mtime = stat_cache_newest(path, 0);
 	if (mtime < 0)
-		return -1;
-	return (mtime != 0);
+		result = -1;
+	else
+		result = (mtime != 0);
+	trace(("return %d\n", result));
+	trace(("}\n"));
+	return result;
+}
+
+
+int
+os_exists_dir(path)
+	string_ty	*path;
+{
+	struct stat	st;
+	int		result;
+
+	trace(("os_exists_symlink(path = \"%s\")\n{\n", path->str_text));
+#ifdef S_ISLNK
+	result = lstat(path->str_text, &st);
+#else
+	result = stat(path->str_text, &st);
+#endif
+	result = (result == 0) && S_ISDIR(st.st_mode);
+	trace(("return %d\n", result));
+	trace(("}\n"));
+	return result;
 }
 
 
@@ -883,317 +921,6 @@ os_accdir()
 }
 
 
-/*
- * NAME
- *	os_curdir - full current directory path
- *
- * SYNOPSIS
- *	string_ty *os_curdir(void);
- *
- * DESCRIPTION
- *	Os_curdir is used to determine the full pathname
- *	of the current directory.
- *
- * RETURNS
- *	A pointer to a string in dynamic memory is returned.
- *	A null pointer is returned on error.
- *
- * CAVEAT
- *	Use str_free() when you are done with the value returned.
- */
-
-string_ty *
-os_curdir()
-{
-	static string_ty	*s;
-
-	if (!s)
-	{
-		char		buffer[1200];
-
-		if (!getcwd(buffer, sizeof(buffer)))
-		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			error_intl(scp, i18n("getcwd: $errno"));
-			sub_context_delete(scp);
-			option_set_errors();
-			return 0;
-		}
-		s = str_from_c(buffer);
-	}
-	return str_copy(s);
-}
-
-
-/*
- * NAME
- *	os_pathname - determine full file name
- *
- * SYNOPSIS
- *	string_ty *os_pathname(string_ty *path);
- *
- * DESCRIPTION
- *	Os_pathname is used to determine the full path name
- *	of a partial path given.
- *
- * RETURNS
- *	pointer to dynamically allocated string.
- *
- * CAVEAT
- *	Use str_free() when you are done with the value returned.
- */
-
-string_ty *
-os_pathname(path)
-	string_ty	*path;
-{
-	static char	*tmp;
-	static size_t	tmplen;
-	static size_t	ipos;
-	static size_t	opos;
-	int		c;
-	int		found;
-#ifdef S_IFLNK
-	char		linkval[2000];
-	int		nbytes;
-	string_list_ty	loop;
-	string_ty	*s;
-#endif
-
-	/*
-	 * Change relative pathnames to absolute
-	 */
-	trace(("os_pathname(path = %08lX)\n{\n"/*}*/, path));
-	trace_string(path->str_text);
-	if (path->str_text[0] != '/')
-	{
-		string_ty	*cwd;
-
-		cwd = os_curdir();
-		assert(cwd);
-		path = str_format("%S/%S", cwd, path);
-		str_free(cwd);
-	}
-	else
-		path = str_copy(path);
-	if (!tmp)
-	{
-		tmplen = 200;
-		tmp = mem_alloc(tmplen);
-	}
-
-	/*
-	 * Take kinks out of the pathname
-	 */
-	ipos = 0;
-	opos = 0;
-	found = 0;
-#ifdef S_IFLNK
-	string_list_constructor(&loop);
-#endif
-	while (!found)
-	{
-		/*
-		 * get the next character
-		 */
-		c = path->str_text[ipos];
-		if (c)
-			ipos++;
-		else
-		{
-			found = 1;
-			c = '/';
-		}
-
-		/*
-		 * remember the normal characters
-		 * until get to slash
-		 */
-		if (c != '/')
-			goto remember;
-
-		/*
-		 * leave root alone
-		 */
-		if (!opos)
-			goto remember;
-
-		/*
-		 * "/.." -> "/"
-		 */
-		if (opos == 3 && tmp[1] == '.' && tmp[2] == '.')
-		{
-			opos = 1;
-			continue;
-		}
-
-		/*
-		 * "a//" -> "a/"
-		 */
-		if (tmp[opos - 1] == '/')
-			continue;
-
-		/*
-		 * "a/./" -> "a/"
-		 */
-		if (opos >= 2 && tmp[opos - 1] == '.' && tmp[opos - 2] == '/')
-		{
-			opos--;
-			continue;
-		}
-
-		/*
-		 * "a/b/../" -> "a/"
-		 */
-		if
-		(
-			opos > 3
-		&&
-			tmp[opos - 1] == '.'
-		&&
-			tmp[opos - 2] == '.'
-		&&
-			tmp[opos - 3] == '/'
-		)
-		{
-			opos -= 4;
-			assert(opos > 0);
-			while (tmp[opos - 1] != '/')
-				opos--;
-			continue;
-		}
-
-		/*
-		 * see if the path so far is a symbolic link
-		 */
-#ifdef S_IFLNK
-		s = str_n_from_c(tmp, opos);
-		if (string_list_member(&loop, s))
-		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_var_set(scp, "File_Name", "%S", s);
-			fatal_intl
-			(
-				scp,
-			       i18n("symbolic link loop \"$filename\" detected")
-			);
-			/* NOTREACHED */
-		}
-		nbytes = readlink(s->str_text, linkval, sizeof(linkval) - 1);
-		if (nbytes < 0)
-		{
-			/*
-			 * probably not a symbolic link
-			 */
-			if
-			(
-				errno != ENXIO
-			&&
-				errno != EINVAL
-			&&
-				errno != ENOENT
-			)
-			{
-				sub_context_ty	*scp;
-
-				scp = sub_context_new();
-				sub_errno_set(scp);
-				sub_var_set(scp, "File_Name", "%S", s);
-				fatal_intl
-				(
-					scp,
-					i18n("readlink $filename: $errno")
-				);
-				/* NOTREACHED */
-			}
-			not_a_symlink:
-			str_free(s);
-		}
-		else
-		{
-			string_ty	*newpath;
-
-			if (nbytes == 0)
-			{
-				sub_context_ty	*scp;
-
-				scp = sub_context_new();
-				sub_var_set(scp, "File_Name", "%S", s);
-				fatal_intl
-				(
-					scp,
-				    i18n("readlink \"$filename\" returned \"\"")
-				);
-				/* NOTREACHED */
-			}
-			linkval[nbytes] = 0;
-
-			/*
-			 * ignore auto-mounter temporary mount links,
-			 * they will change with each mount.
-			 */
-			if (!strncmp(linkval, "/tmp_mnt/", 9))
-				goto not_a_symlink;
-
-			string_list_append(&loop, s);
-			str_free(s);
-			if (linkval[0] == '/')
-				tmp[1] = 0;
-			else
-			{
-				while (tmp[opos - 1] != '/')
-					opos--;
-				tmp[opos] = 0;
-			}
-			newpath =
-				str_format
-				(
-					"%s/%s/%s",
-					tmp,
-					linkval,
-					path->str_text + ipos
-				);
-			str_free(path);
-			path = newpath;
-			ipos = 0;
-			opos = 0;
-			found = 0;
-			continue;
-		}
-#endif
-
-		/*
-		 * keep the slash
-		 */
-		remember:
-		if (opos >= tmplen)
-		{
-			tmplen += 100;
-			tmp = mem_change_size(tmp, tmplen);
-		}
-		tmp[opos++] = c;
-	}
-#ifdef S_IFLNK
-	string_list_destructor(&loop);
-#endif
-	str_free(path);
-	assert(opos >= 1);
-	assert(tmp[0] == '/');
-	assert(tmp[opos - 1] == '/');
-	if (opos >= 2)
-		opos--;
-	path = str_n_from_c(tmp, opos);
-	trace_string(path->str_text);
-	trace((/*{*/"}\n"));
-	return path;
-}
-
-
 /*
  * NAME
  *	os_entryname - take path apart
@@ -1218,7 +945,7 @@ os_entryname(path)
 {
 	char		*cp;
 
-	trace(("os_entryname(path = %08lX) entry", path));
+	trace(("os_entryname(path = %08lX)\n{\n", path));
 	trace_string(path->str_text);
 	cp = strrchr(path->str_text, '/');
 	if (cp)
@@ -1450,6 +1177,22 @@ os_mkdir(p, echo)
 			string_ty	*tmp;
 
 			tmp = str_n_from_c(p->str_text, cp - p->str_text);
+#if defined(_SGI_SOURCE) && !defined(_ST_PAD4SZ)
+			if (os_exists_dir(tmp))
+			{
+		        	/*
+				 * Do nothing.
+				 *
+				 * SGI's IRIX 5.2 has a bug: it tests
+				 * for permissions before it checks
+				 * for existence.  Thus, you can get a
+				 * permission denied error for a directory
+				 * you couldn't possibly have tried to
+				 * create, because it already existed.
+				 */
+			}
+			else
+#endif
 			if (mkdir(tmp->str_text, 0777) < 0)
 			{
 				if (errno != EEXIST)
