@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -34,9 +34,11 @@ int os_touch _((string_ty *));
 int os_execute _((string_list_ty *cmd, string_ty *input, int errok));
 int os_exists _((string_ty *));
 int os_exists_symlink _((string_ty *));
+int os_exists_dir _((string_ty *));
 string_ty *os_accdir _((void));
 string_ty *os_curdir _((void));
 string_ty *os_dirname _((string_ty *));
+string_ty *os_dirname_relative _((string_ty *));
 string_ty *os_entryname _((string_ty *));
 string_ty *os_pathname _((string_ty *));
 int os_legal_path _((string_ty *));
@@ -44,5 +46,6 @@ int os_delete _((string_ty *path, int echo));
 int os_clear_stat _((string_ty *));
 int exit_status _((char *cmd, int status, int errok));
 int os_mkdir _((string_ty *path, int echo));
+string_ty *os_path_cat _((string_ty *, string_ty *));
 
 #endif /* OS_H */
