@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,15 @@
  * MANIFEST: functions to manipulate the stat cache
  */
 
-#include <errno.h>
 #include <ac/string.h>
+#include <ac/errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <archive.h>
 #include <error_intl.h>
-#include <fngrprnt.h>
+#include <fingerprint.h>
+#include <fingerprint/value.h>
 #include <mem.h>
 #include <option.h>
 #include <stat.cache.h>
@@ -45,6 +46,18 @@ struct cache_ty
 static symtab_ty *symtab[2];
 
 
+/*
+ * NAME
+ *	init
+ *
+ * SYNOPSIS
+ *	void init(int follow_links);
+ *
+ * DESCRIPTION
+ *	The init function is used to init one of the two databases.
+ *	It creates the symbol table, and sets its reap function.
+ */
+
 static void init _((int));
 
 static void
@@ -61,6 +74,18 @@ init(n)
 }
 
 
+/*
+ * NAME
+ *	mem_copy_cache
+ *
+ * SYNOPSIS
+ *	cache_ty *mem_copy_cache(cache_ty *);
+ *
+ * DESCRIPTION
+ *	The mem_copy_cache creates a copy of a cache in dynamic memory.
+ *	(New with copy constructor.)
+ */
+
 static cache_ty *mem_copy_cache _((cache_ty *));
 
 static cache_ty *
@@ -75,6 +100,193 @@ mem_copy_cache(st)
 }
 
 
+/*
+ * NAME
+ *	stat_cache_fingerprint
+ *
+ *	stat_cache_fingerprint - mangle the cache_ty value
+ *
+ * SYNOPSIS
+ *	void stat_cache_fingerprint(string_ty *path, struct cache_ty *);
+ *
+ * DESCRIPTION
+ *	The stat_cache_fingerprint function is used to mangle a cache_ty
+ *	based on the file fingerprint information.  It also updates the
+ *	file fingerprint database if necessary.
+ */
+
+static void stat_cache_fingerprint _((string_ty *, cache_ty *));
+
+static void
+stat_cache_fingerprint(path, cp)
+	string_ty	*path;
+	cache_ty	*cp;
+{
+	fp_value_ty	*fp;
+
+	assert(cp->newest == cp->oldest);
+	fp = fp_search(path);
+	if (fp)
+	{
+		/*
+		 * we have seen this file before
+		 */
+		if (fp->newest == cp->newest)
+		{
+			/*
+			 * file not modified since last seen
+			 */
+			cp->oldest = fp->oldest;
+		}
+		else if (cp->newest < fp->newest)
+		{
+			string_ty	*s;
+
+			/*
+			 * it travelled backwards in time
+			 *	(e.g. ClearCase or 3DFS or unionfs, etc)
+			 * since we last saw it.  (Remember the
+			 * fingerprint, but it plays no part in the
+			 * comaprison.)
+			 */
+			s = fp_fingerprint(path);
+			if (!s)
+				goto fp_not_useful;
+			if (str_equal(fp->fingerprint, s))
+			{
+				cp->newest = fp->newest;
+				cp->oldest = cp->newest;
+			}
+			else
+			{
+				fp_value_ty	data;
+
+				time(&cp->newest);
+				cp->oldest = cp->newest;
+				fp_value_constructor3
+				(
+					&data,
+					cp->oldest,
+					cp->newest,
+					s
+				);
+				fp_assign(path, &data);
+				fp_value_destructor(&data);
+			}
+			str_free(s);
+		}
+		else
+		{
+			string_ty	*s;
+			fp_value_ty	data;
+
+			/*
+			 * but it has changed
+			 * since we last saw it
+			 */
+			s = fp_fingerprint(path);
+			if (!s)
+				goto fp_not_useful;
+			if (str_equal(fp->fingerprint, s))
+			{
+				/*
+				 * the fingerprint is the same,
+				 * so give the oldest mtime
+				 * known
+				 */
+				fp_value_constructor3
+				(
+					&data,
+					fp->oldest,
+					cp->newest,
+					s
+				);
+				str_free(s);
+				cp->oldest = fp->oldest;
+				if (option_test(OPTION_REASON))
+				{
+					struct tm	*tm;
+					sub_context_ty	*scp;
+
+					tm = localtime(&data.newest);
+					scp = sub_context_new();
+					sub_var_set
+					(
+						scp,
+						"File_Name",
+						"%S",
+						path
+					);
+					sub_var_set
+					(
+						scp,
+						"Number",
+				 "%4d/%02d/%02d.%02d:%02d:%02d",
+					     1900 + tm->tm_year,
+						tm->tm_mon + 1,
+						tm->tm_mday,
+						tm->tm_hour,
+						tm->tm_min,
+						tm->tm_sec
+					);
+					error_intl
+					(
+						scp,
+	  i18n("mtime(\"$filename\") was $number until fingerprinting (reason)")
+					);
+				}
+			}
+			else
+			{
+				/*
+				 * the fingerprint differs
+				 * do not lie about mtime
+				 */
+				fp_value_constructor3
+				(
+					&data,
+					cp->newest,
+					cp->newest,
+					s
+				);
+				str_free(s);
+			}
+			fp_assign(path, &data);
+			fp_value_destructor(&data);
+		}
+	}
+	else
+	{
+		string_ty	*s;
+
+		/*
+		 * never fingerprinted this file before
+		 */
+		s = fp_fingerprint(path);
+		if (!s)
+		{
+			fp_not_useful:
+			fp_delete(path);
+		}
+		else
+		{
+			fp_value_ty	data;
+
+			fp_value_constructor3
+			(
+				&data,
+				cp->newest,
+				cp->newest,
+				s
+			);
+			str_free(s);
+			fp_assign(path, &data);
+			fp_value_destructor(&data);
+		}
+	}
+}
+
+
 /*
  * NAME
  *	stat_cache - stat() with caching
@@ -106,21 +318,22 @@ stat_cache(path, cp, follow_links)
 	cache_ty	*data_p;
 	int		err;
 	struct stat	st;
-	sub_context_ty	*scp;
 
 	/*
 	 * if we have previously stat()ed this file,
 	 * return old information
 	 */
-	trace(("stat_cache(path = \"%s\")\n{\n"/*}*/, path->str_text));
+	trace(("stat_cache(path = \"%s\", lnk = %d)\n{\n",
+		path->str_text, follow_links));
 	if (!symtab[follow_links])
 		init(follow_links);
 	data_p = symtab_query(symtab[follow_links], path);
 	if (data_p)
 	{
 		*cp = *data_p;
+		trace(("got it from the cache\n"));
 		trace(("return 0;\n"));
-		trace((/*{*/"}\n"));
+		trace(("}\n"));
 		return 0;
 	}
 
@@ -128,7 +341,7 @@ stat_cache(path, cp, follow_links)
 	 * new file, perform stat() for the first time
 	 */
 	trace(("stat(\"%s\")\n", path->str_text));
-#ifdef S_IFLNK
+#if defined(S_IFLNK) || defined(S_ISLNK)
 	if (!follow_links)
 		err = lstat(path->str_text, &st);
 	else
@@ -151,13 +364,9 @@ stat_cache(path, cp, follow_links)
 			break;
 
 		default:
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", path);
-			error_intl(scp, i18n("stat $filename: $errno"));
-			sub_context_delete(scp);
+			error_intl_stat(path->str_text);
 			trace(("return -1;\n"));
-			trace((/*{*/"}\n"));
+			trace(("}\n"));
 			return -1;
 		}
 
@@ -168,8 +377,6 @@ stat_cache(path, cp, follow_links)
 	}
 	else
 	{
-		fp_ty	*fp;
-
 		/*
 		 * make sure the times of existing files
 		 * are always positive
@@ -183,108 +390,7 @@ stat_cache(path, cp, follow_links)
 		 * see if we have its fingerprint on file
 		 */
 		if (option_test(OPTION_FINGERPRINT))
-		{
-			fp = fp_search(path);
-			if (fp)
-			{
-				/*
-				 * we have seen this file before
-				 */
-				if (fp->newest != cp->newest)
-				{
-					fp_ty	data;
-
-					/*
-					 * but it has changed
-					 * since we last saw it
-					 */
-					data.fingerprint = fp_fingerprint(path);
-					if (!data.fingerprint)
-						goto fp_not_useful;
-					if (str_equal(fp->fingerprint, data.fingerprint))
-					{
-						/*
-						 * the fingerprint is the same,
-						 * so give the oldest mtime
-						 * known
-						 */
-						data.oldest = fp->oldest;
-						data.newest = cp->newest;
-						cp->oldest = fp->oldest;
-						if (option_test(OPTION_REASON))
-						{
-							struct tm	*tm;
-
-							tm = localtime(&data.newest);
-							scp = sub_context_new();
-							sub_var_set
-							(
-								scp,
-								"File_Name",
-								"%S",
-								path
-							);
-							sub_var_set
-							(
-								scp,
-								"Number",
-						 "%4d/%02d/%02d.%02d:%02d:%02d",
-							     1900 + tm->tm_year,
-								tm->tm_mon + 1,
-								tm->tm_mday,
-								tm->tm_hour,
-								tm->tm_min,
-								tm->tm_sec
-							);
-							error_intl
-							(
-								scp,
-	  i18n("mtime(\"$filename\") was $number until fingerprinting (reason)")
-							);
-						}
-					}
-					else
-					{
-						/*
-						 * the fingerprint differs
-						 * do not lie about mtime
-						 */
-						data.oldest = cp->newest;
-						data.newest = cp->newest;
-					}
-					fp_assign(path, &data);
-					str_free(data.fingerprint);
-				}
-				else
-				{
-					/*
-					 * file not modified since last seen
-					 */
-					cp->oldest = fp->oldest;
-				}
-			}
-			else
-			{
-				fp_ty	data;
-
-				/*
-				 * never fingerprinted this file before
-				 */
-				data.oldest = cp->newest;
-				data.newest = cp->newest;
-				data.fingerprint = fp_fingerprint(path);
-				if (!data.fingerprint)
-				{
-					fp_not_useful:
-					fp_delete(path);
-				}
-				else
-				{
-					fp_assign(path, &data);
-					str_free(data.fingerprint);
-				}
-			}
-		}
+			stat_cache_fingerprint(path, cp);
 	}
 
 	/*
@@ -292,11 +398,30 @@ stat_cache(path, cp, follow_links)
 	 */
 	symtab_assign(symtab[follow_links], path, mem_copy_cache(cp));
 	trace(("return 0;\n"));
-	trace((/*{*/"}\n"));
+	trace(("}\n"));
 	return 0;
 }
 
 
+/*
+ * NAME
+ *	stat_cache_newest
+ *
+ * SYNOPSIS
+ *	time_t stat_cache_newest(string_ty *path, int follow_links);
+ *
+ * DESCRIPTION
+ *	The stat_cahe_newest function is used to obtain the upper bound of
+ *	the time range for which the file has had its current contents.
+ *
+ * RETURNS
+ *	time_t; the time the file was last modified, (time_t)0 if the
+ *	file does not exist.
+ *
+ * CAVEAT
+ *	All errors except ENOENT result in a fatal error message.
+ */
+
 time_t
 stat_cache_newest(path, follow_links)
 	string_ty	*path;
@@ -305,8 +430,14 @@ stat_cache_newest(path, follow_links)
 	cache_ty	cache;
 	sub_context_ty	*scp;
 
+	trace(("stat_cache_newest(path = \"%s\", lnk = %d)\n{\n",
+		(long)path->str_text, follow_links));
 	if (stat_cache(path, &cache, !!follow_links))
+	{
+		trace(("Bzzt!\n"));
+		trace(("}\n"));
 		return -1;
+	}
 
 	/*
 	 * trace the last-modified time
@@ -366,10 +497,31 @@ stat_cache_newest(path, follow_links)
 			}
 		}
 	}
+	trace(("return (%ld) %s", (long)cache.newest, ctime(&cache.newest)));
+	trace(("}\n"));
 	return cache.newest;
 }
 
 
+/*
+ * NAME
+ *	stat_cache_oldest
+ *
+ * SYNOPSIS
+ *	time_t stat_cache_oldest(string_ty *path, int follow_links);
+ *
+ * DESCRIPTION
+ *	The stat_cahe_oldest function is used to obtain the lower bound of
+ *	the time range for which the file has had its current contents.
+ *
+ * RETURNS
+ *	time_t; the time the file was last modified, (time_t)0 if the
+ *	file does not exist.
+ *
+ * CAVEAT
+ *	All errors except ENOENT result in a fatal error message.
+ */
+
 time_t
 stat_cache_oldest(path, follow_links)
 	string_ty	*path;
@@ -378,8 +530,14 @@ stat_cache_oldest(path, follow_links)
 	cache_ty	cache;
 	sub_context_ty	*scp;
 
+	trace(("stat_cache_oldest(path = \"%s\", lnk = %d)\n{\n",
+		(long)path->str_text, follow_links));
 	if (stat_cache(path, &cache, !!follow_links))
+	{
+		trace(("Bzzt!\n"));
+		trace(("}\n"));
 		return -1;
+	}
 
 	/*
 	 * trace the last-modified time
@@ -439,6 +597,8 @@ stat_cache_oldest(path, follow_links)
 			}
 		}
 	}
+	trace(("return (%ld) %s", (long)cache.oldest, ctime(&cache.oldest)));
+	trace(("}\n"));
 	return cache.oldest;
 }
 
@@ -491,18 +651,22 @@ stat_cache_set(path, when, fp2)
 	 */
 	if (fp2 && option_test(OPTION_FINGERPRINT))
 	{
-		fp_ty		*fp;
+		fp_value_ty	*fp;
 
 		fp = fp_search(path);
 		if (fp)
 		{
-			fp_ty		data;
+			fp_value_ty	data;
 
-			data.oldest = fp->oldest;
-			data.newest = when;
-			data.fingerprint = str_copy(fp->fingerprint);
+			fp_value_constructor3
+			(
+				&data,
+				fp->oldest, 
+				when,
+				fp->fingerprint
+			);
 			fp_assign(path, &data);
-			str_free(data.fingerprint);
+			fp_value_destructor(&data);
 		}
 	}
 
