@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,8 +20,8 @@
  * MANIFEST: operating system start point, and command line argument parsing
  */
 
-#include <errno.h>
-#include <stdio.h>
+#include <ac/errno.h>
+#include <ac/stdio.h>
 #include <ac/stdlib.h>
 
 #include <arglex.h>
