@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1992, 1993, 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1992, 1993, 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  */
 
 #include <ac/stddef.h>
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/string.h>
 #include <ac/stdlib.h>
 
@@ -126,12 +126,9 @@ main(argc, argv)
 		case arglex_token_report:
 			if (report)
 			{
-				sub_context_ty	*scp;
-
 				duplicate:
-				scp = sub_context_new();
-				sub_var_set(scp, "Name", "%s", arglex_value.alv_string);
-				fatal_intl(scp, i18n("duplicate \"$name\" option"));
+				arg_duplicate_cur(usage);
+				/* NOTREACHED */
 			}
 			report++;
 			break;
@@ -141,11 +138,8 @@ main(argc, argv)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
 			{
-				sub_context_ty	*scp;
-
-				scp = sub_context_new();
-				sub_var_set(scp, "Name", "%s", arglex_token_name(arglex_token_access));
-				fatal_intl(scp, i18n("$name needs string"));
+				arg_needs_string(arglex_token_access, usage);
+				/* NOTREACHED */
 			}
 			atime_set = 1;
 			atime = date_scan(arglex_value.alv_string);
@@ -168,11 +162,8 @@ main(argc, argv)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
 			{
-				sub_context_ty	*scp;
-
-				scp = sub_context_new();
-				sub_var_set(scp, "Name", "%s", arglex_token_name(arglex_token_modify));
-				fatal_intl(scp, i18n("$name needs string"));
+				arg_needs_string(arglex_token_modify, usage);
+				/* NOTREACHED */
 			}
 			mtime_set = 1;
 			mtime = date_scan(arglex_value.alv_string);
@@ -207,14 +198,7 @@ main(argc, argv)
 
 		s = filename.string[j];
 		if (stat(s->str_text, &st))
-		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", s);
-			fatal_intl(scp, i18n("stat $filename: $errno"));
-		}
+			fatal_intl_stat(s->str_text);
 		if (mtime_set)
 			ut.modtime = mtime;
 		else
@@ -228,6 +212,7 @@ main(argc, argv)
 			sub_context_ty	*scp;
 
 			scp = sub_context_new();
+			sub_errno_set(scp);
 			sub_var_set(scp, "File_Name", "%S", s);
 			fatal_intl(scp, i18n("utime $filename: $errno"));
 		}
