@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	cook - file construction tool
-#	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998 Peter Miller;
+#	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #
 # MANIFEST: shell script to generate Makefile file
 #
-clean_files="core y.tab.c y.tab.h y.output .bin .bindir common/lib.a"
+clean_files="core y.tab.c y.tab.h y.output .bin .bindir common/lib.\$(LIBEXT)"
 
 #
 # list of progams to be linked and installed
@@ -75,17 +75,17 @@ do
 	*.y)
 		dir=`echo $file | sed 's|^\([^/]*\)/.*$|\1|'`
 		stem=`echo $file | sed 's/\.y$//'`
-		eval "${dir}_files=\"\$${dir}_files ${stem}.gen.o\""
+		eval "${dir}_files=\"\$${dir}_files ${stem}.gen.\\\$(OBJEXT)\""
 		clean_files="$clean_files ${stem}.gen.c ${stem}.gen.h \
-${stem}.gen.o"
+${stem}.gen.\$(OBJEXT)"
 		remember_prog $dir
 		;;
 
 	*.c)
 		dir=`echo $file | sed 's|^\([^/]*\)/.*$|\1|'`
 		stem=`echo $file | sed 's/\.c$//'`
-		eval "${dir}_files=\"\$${dir}_files ${stem}.o\""
-		clean_files="$clean_files ${stem}.o"
+		eval "${dir}_files=\"\$${dir}_files ${stem}.\\\$(OBJEXT)\""
+		clean_files="$clean_files ${stem}.\$(OBJEXT)"
 		remember_prog $dir
 		;;
 
@@ -163,18 +163,18 @@ do
 	if test "$prog" = common; then continue; fi
 
 	echo ""
-	echo "bin/${prog}: \$(${prog}_obj) common/lib.a .bin"
-	echo "	\$(CC) \$(LDFLAGS) -o \$@ \$(${prog}_obj) common/lib.a \$(LIBS)"
+	echo "bin/${prog}\$(EXEEXT): \$(${prog}_obj) common/lib.\$(LIBEXT) .bin"
+	echo "	\$(CC) \$(LDFLAGS) -o \$@ \$(${prog}_obj) common/lib.\$(LIBEXT) \$(LIBS)"
 
-	all="${all} bin/${prog}"
+	all="${all} bin/${prog}\$(EXEEXT)"
 
 	if test "$prog" = fstrcmp; then continue; fi
 
 	echo ""
-	echo "\$(bindir)/${prog}: bin/${prog} .bindir"
-	echo "	\$(INSTALL_PROGRAM) bin/${prog} \$@"
+	echo "\$(bindir)/\$(PROGRAM_PREFIX)${prog}\$(PROGRAM_SUFFIX)\$(EXEEXT): bin/${prog}\$(EXEEXT) .bindir"
+	echo "	\$(INSTALL_PROGRAM) bin/${prog}\$(EXEEXT) \$@"
 
-	install_bin="${install_bin} \$(bindir)/${prog}"
+	install_bin="${install_bin} \$(bindir)/\$(PROGRAM_PREFIX)${prog}\$(PROGRAM_SUFFIX)\$(EXEEXT)"
 done
 
 echo ""
@@ -184,7 +184,7 @@ echo "#"
 echo "all:" ${all} po groff_all
 
 echo ""
-echo "common/lib.a: \$(common_obj)"
+echo "common/lib.\$(LIBEXT): \$(common_obj)"
 echo "	rm -f \$@"
 echo "	\$(AR) qc \$@ \$(common_obj)"
 echo "	\$(RANLIB) \$@"
@@ -193,13 +193,13 @@ echo ""
 echo ".bin:"
 echo "	-mkdir bin"
 echo "	-chmod 0755 bin"
-echo "	@-test -d .bin && touch .bin"
+echo "	@-test -d bin && touch .bin"
 echo "	@sleep 1"
 
 echo ""
 echo ".bindir:"
 echo "	-\$(INSTALL) -m 0755 -d \$(bindir)"
-echo "	@-test -d .bindir && touch .bindir"
+echo "	@-test -d \$(bindir) && touch .bindir"
 echo "	@sleep 1"
 
 echo ""
@@ -208,12 +208,14 @@ echo "	@echo Passed All Tests"
 
 echo ""
 echo "po_files_yes =" $po_files
+echo ""
 echo "po_files_no ="
 echo ""
 echo "po: \$(po_files_@po_files@)"
 
 echo ""
 echo "groff_all_yes =" $groff_all
+echo ""
 echo "groff_all_no ="
 echo ""
 echo "groff_all: \$(groff_all_@have_groff@)"
@@ -253,6 +255,7 @@ echo "install-man:" $man_files
 
 echo ""
 echo "po_install_files_yes =" $po_install_files
+echo ""
 echo "po_install_files_no ="
 
 echo ""
@@ -260,6 +263,7 @@ echo "install-po: \$(po_install_files_@po_files@)"
 
 echo ""
 echo "groff_install_yes =" $groff_install
+echo ""
 echo "groff_install_no ="
 
 echo ""
