@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	cook - file construction tool
-#	Copyright (C) 1998 Peter Miller;
+#	Copyright (C) 1998, 1999 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,27 @@
 #
 # MANIFEST: shell script to build the RPM package
 #
+case $# in
+3)
+	icon=$3
+	;;
+2)
+	icon=
+	;;
+*)
+	echo "Usage: $0 <tmp-dir> <tar-ball>" 1>&2
+	exit 1
+	;;
+esac
 tmp=$1
 tarball=$2
 
 mkdir -p $tmp/BUILD $tmp/BUILD_ROOT $tmp/RPMS/i386 \
 	$tmp/SOURCES $tmp/SPECS $tmp/SRPMS
 
+# copy the icon, if one was given
+test "z$icon" != "z" && cp $icon $tmp/SOURCES/.
+
 here=`pwd`/$tmp
 cat > $tmp/rpmrc << fubar
 builddir: $here/BUILD
