@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  * MANIFEST: functions to isolate operating system interface
  */
 
+#include <ac/errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <errno.h>
 
 #include <error_intl.h>
 #include <os.h>
@@ -42,7 +42,6 @@ int
 os_exists(filename)
 	char		*filename;
 {
-	sub_context_ty	*scp;
 	struct stat	st;
 
 	if (stat(filename, &st))
@@ -54,10 +53,7 @@ os_exists(filename)
 			break;
 
 		default:
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", filename);
-			fatal_intl(scp, i18n("stat $filename: $errno"));
+			fatal_intl_stat(filename);
 			/* NOTREACHED */
 		}
 		return 0;
