@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  * MANIFEST: functions to emit the result
  */
 
-#include <stdio.h>
+#include <ac/stdio.h>
 
 #include <emit.h>
 #include <error_intl.h>
@@ -47,14 +47,7 @@ emit_open(s)
 	assert(!output);
 	if (s)
 	{
-		output = fopen(s, "w");
-		if (!output)
-		{
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", s);
-			fatal_intl(scp, i18n("open $filename: $errno"));
-		}
+		output = fopen_and_check(s, "w");
 		filename = str_from_c(s);
 	}
 	else
@@ -73,24 +66,11 @@ emit_open(s)
 void
 emit_close()
 {
-	sub_context_ty	*scp;
-
 	assert(output);
 	emit_bol();
-	if (fflush(output))
-	{
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", filename);
-		fatal_intl(scp, i18n("write $filename: $errno"));
-	}
-	if (output != stdout && fclose(output))
-	{
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", filename);
-		fatal_intl(scp, i18n("close $filename: $errno"));
-	}
+	fflush_and_check(output, filename->str_text);
+	if (output != stdout)
+		fclose_and_check(output, filename->str_text);
 	str_free(filename);
 	filename = 0;
 	output = 0;
@@ -154,13 +134,8 @@ emit_char(c)
 	}
 	if (putc(c, output) == EOF)
 	{
-		sub_context_ty	*scp;
-
 		bomb:
-		scp = sub_context_new();
-		sub_errno_set(scp);
-		sub_var_set(scp, "File_Name", "%S", filename);
-		fatal_intl(scp, i18n("write $filename: $errno"));
+		fatal_intl_write(filename->str_text);
 	}
 	++icol;
 	++ocol;
@@ -214,15 +189,7 @@ emit_line_number(lino, fn)
 	{
 		emit_bol();
 		fprintf(output, "#line %ld \"%s\"\n", lino, fn->str_text);
-		if (ferror(output))
-		{
-			sub_context_ty	*scp;
-
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%S", filename);
-			fatal_intl(scp, i18n("write $filename: $errno"));
-		}
+		fflush_and_check(output, filename->str_text);
 	}
 	else if (!str_equal(old_fn, fn) || lino > (old_ln + (ocol != 0)))
 	{
