@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994, 1997, 1998 Peter Miller;
+ *	Copyright (C) 1994, 1997, 1998, 1999 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,13 @@
  */
 
 #include <ac/ctype.h>
-#include <ac/stdio.h>
 #include <ac/string.h>
 #include <ac/stdarg.h>
 
 #include <blob.h>
 #include <error_intl.h>
+#include <input/file_text.h>
+#include <input/stdin.h>
 #include <lex.h>
 #include <mem.h>
 #include <stmt.h>
@@ -34,8 +35,7 @@
 #include <trace.h>
 #include <gram.gen.h> /* must be last */
 
-static string_ty *file_name;
-static FILE	*input;
+static input_ty	*input;
 static long	line_number;
 static int	error_count;
 static int	bol;
@@ -52,14 +52,14 @@ static void notify _((void));
 static void
 notify()
 {
-	if (!file_name)
+	if (!input)
 		return;
 	if (++error_count >= 20)
 	{
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "File_Name", "%S", file_name);
+		sub_var_set(scp, "File_Name", "%S", input_filename(input));
 		fatal_intl(scp, i18n("$filename: too many fatal errors"));
 	}
 }
@@ -74,26 +74,15 @@ lex_open(s)
 	blob_error_notify(notify);
 	if (s)
 	{
-		input = fopen(s, "r");
-		if (!input)
-		{
-			sub_context_ty	*scp;
+		string_ty	*filename;
 
-			scp = sub_context_new();
-			sub_errno_set(scp);
-			sub_var_set(scp, "File_Name", "%s", s);
-			fatal_intl(scp, i18n("open $filename: $errno"));
-		}
-		file_name = str_from_c(s);
+		filename = str_from_c(s);
+		input = input_file_text_open(filename);
+		str_free(filename);
 	}
 	else
 	{
-		sub_context_ty	*scp;
-
-		scp = sub_context_new();
-		file_name = subst_intl(scp, i18n("standard input"));
-		sub_context_delete(scp);
-		input = stdin;
+		input = input_file_text_open((string_ty *)0);
 	}
 	line_number = 1;
 	bol = 1;
@@ -114,16 +103,13 @@ lex_close()
 		sub_context_ty	*scp;
 
 		scp = sub_context_new();
-		sub_var_set(scp, "File_Name", "%S", file_name);
+		sub_var_set(scp, "File_Name", "%S", input_filename(input));
 		sub_var_set(scp, "Number", "%d", error_count);
 		sub_var_optional(scp, "Number");
 		fatal_intl(scp, i18n("$filename: found $number fatal errors"));
 	}
-	if (input != stdin)
-		fclose(input);
+	input_delete(input);
 	input = 0;
-	str_free(file_name);
-	file_name = 0;
 	line_number = 0;
 	bol = 0;
 	first = 0;
@@ -161,7 +147,7 @@ lex_error(scp, s)
 	while (len > 0 && isspace(buffer->str_text[len - 1]))
 		--len;
 	/* re-use substitution context */
-	sub_var_set(scp, "File_Name", "%S", file_name);
+	sub_var_set(scp, "File_Name", "%S", input_filename(input));
 	sub_var_set(scp, "Number", "%d", line_number);
 	sub_var_set(scp, "MeSsaGe", "%.*S", len, buffer);
 	str_free(buffer);
@@ -183,19 +169,10 @@ byte()
 	assert(input);
 	for (;;)
 	{
-		c = getc(input);
+		c = input_getc(input);
 		switch (c)
 		{
-		case EOF:
-			if (ferror(input))
-			{
-				sub_context_ty	*scp;
-
-				scp = sub_context_new();
-				sub_errno_set(scp);
-				sub_var_set(scp, "File_Name", "%S", file_name);
-				fatal_intl(scp, i18n("read $filename: $errno"));
-			}
+		case INPUT_EOF:
 			bol = 1;
 			break;
 		
@@ -216,14 +193,14 @@ byte()
 			break;
 
 		case '\\':
-			c = getc(input);
+			c = input_getc(input);
 			if (c == '\n')
 			{
 				++line_number;
 				continue;
 			}
-			if (c != EOF)
-				ungetc(c, input);
+			if (c != INPUT_EOF)
+				input_ungetc(input, c);
 			c = '\\';
 			/* fall through... */
 
@@ -245,16 +222,15 @@ byte_undo(c)
 {
 	switch (c)
 	{
-	case EOF:
+	case INPUT_EOF:
 		break;
 
 	case '\n':
 		--line_number;
-		ungetc(c, input);
-		break;
+		/* fall through... */
 	
 	default:
-		ungetc(c, input);
+		input_ungetc(input, c);
 		break;
 	}
 }
@@ -304,7 +280,7 @@ gram_trace(s sva_last)
 		trace_printf
 		(
 			"%s: %ld: %s\n",
-			file_name->str_text,
+			input_filename(input)->str_text,
 			line_number,
 			line
 		);
@@ -336,7 +312,7 @@ gram_trace2(garbage, s sva_last)
 		trace_printf
 		(
 			"%s: %ld: %s\n",
-			file_name->str_text,
+			input_filename(input)->str_text,
 			line_number,
 			line
 		);
@@ -473,7 +449,7 @@ gram_lex()
 		c = byte();
 		switch (c)
 		{
-		case EOF:
+		case INPUT_EOF:
 			token = 0;
 			goto done;
 
@@ -486,7 +462,7 @@ gram_lex()
 				c = byte();
 				switch (c)
 				{
-				case EOF:
+				case INPUT_EOF:
 				case '\n':
 					break;
 
@@ -507,7 +483,7 @@ gram_lex()
 				break;
 			}
 			gram_lval.lv_line =
-				blob_alloc(sa_close(), file_name, linum);
+				blob_alloc(sa_close(), input_filename(input), linum);
 			token = COMMAND;
 			goto done;
 
@@ -520,7 +496,7 @@ gram_lex()
 				c = byte();
 				switch (c)
 				{
-				case EOF:
+				case INPUT_EOF:
 				case '\n':
 					break;
 
@@ -579,7 +555,7 @@ gram_lex()
 				colon_special = 1;
 			}
 			gram_lval.lv_line =
-				blob_alloc(sa_close(), file_name, linum);
+				blob_alloc(sa_close(), input_filename(input), linum);
 			goto done;
 
 		case ' ':
@@ -647,7 +623,7 @@ gram_lex()
 			{
 				switch (c)
 				{
-				case EOF:
+				case INPUT_EOF:
 				case '\n':
 					break;
 
@@ -739,7 +715,7 @@ gram_lex()
 
 				case IF:
 					gram_lval.lv_line =
-						blob_alloc(s, file_name, linum);
+						blob_alloc(s, input_filename(input), linum);
 					break;
 				
 				case VPATH:
@@ -753,7 +729,7 @@ gram_lex()
 				goto done;
 			}
 			gram_lval.lv_line =
-				blob_alloc(s, file_name, linum);
+				blob_alloc(s, input_filename(input), linum);
 			token = WORD;
 			goto done;
 		}
@@ -777,5 +753,5 @@ blob_ty *
 lex_blob(s)
 	string_ty	*s;
 {
-	return blob_alloc(s, file_name, line_number - first);
+	return blob_alloc(s, input_filename(input), line_number - first);
 }
