@@ -56,15 +56,15 @@ COOK_MESSAGE_LIBRARY=$work/no-such-dir
 export COOK_MESSAGE_LIBRARY
 unset LANG
 
-cat > Howto.cook <<foobar
+cat > Howto.cook << 'foobar'
 test:
 {
-	if [not [in [dir x] [pathname $tmp]]] then fail;
+	if [not [in [dir x] [pathname .]]] then fail;
 	if [not [in [dir x/y] x]] then fail;
 	if [not [in [dir /] /]] then fail;
 	if [not [in [dir /foo] /]] then fail;
 	if [not [in [dir /foo/bar] /foo]] then fail;
-	if [not [in [dirname x] [pathname $tmp]]] then fail;
+	if [not [in [dirname x] [pathname .]]] then fail;
 	if [not [in [dirname x/y] x]] then fail;
 	if [not [in [dirname /] /]] then fail;
 	if [not [in [dirname /foo] /]] then fail;
