@@ -60,11 +60,21 @@ unset LANG
 # note: Linux has a symlink /bin/sh -> /bin/bash
 #
 cat > Howto.cook <<foobar
+exe = ;
+if [in [substr 1 6 [os]] CYGWIN NUTCRA] then
+	exe = .exe;
 test:
 {
 	if [find_command '{s h * t}' ] then fail;
-	if [not [in [find_command sh] /bin/sh /usr/bin/sh /usr/ucb/sh
-		/sbin/sh /bin/bash ] ] then fail;
+	path =
+		[stringset
+			[split ':' [getenv PATH]]
+			/bin /usr/bin /sbin /usr/sbin
+		];
+	/* function print 'SH =' [find_command sh]; */
+	if [not [in [find_command sh]
+		[addsuffix /sh[exe] [path]]
+		] ] then fail;
 }
 foobar
 if test $? -ne 0 ; then fail; fi
