@@ -59,8 +59,18 @@ unset LANG
 cat > Howto.cook <<foobar
 test:
 {
-	if [not [in [pathname .] [pathname $tmp]]] then fail;
-	if [not [in [pathname x] [pathname $tmp/x]]] then fail;
+	if [not [in [substr 1 6 [os]] CYGWIN NUTCRA]] then
+	{
+		/*
+		 * On CygWin32, if you have 
+		 *	mount C:/temp /tmp
+		 * then the relative name will give /temp/...,
+		 * and the absolute name will give /tmp/...
+		 * Just skip this for Cygwin32. 
+		 */
+		if [not [in [pathname .] [pathname $tmp]]] then fail;
+		if [not [in [pathname x] [pathname $tmp/x]]] then fail;
+	}
 	if [not [in [pathname /] /]] then fail;
 	if [not [in [pathname /////usr/bin//////././//.///] /usr/bin]]
 		then fail;
