@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	cook - file construction tool
-#	Copyright (C) 1994, 1997, 1998 Peter Miller;
+#	Copyright (C) 1994, 1997, 1998, 1999 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -70,7 +70,9 @@ test: foo;
 foo: bar
 {
 	if [exists foo] then
-		fail;
+	{
+		fail "This recipe should only be run when foo does not exist, but if foo exists it has the same fingerprint so the test fails.";
+	}
 	cp bar foo;
 }
 fubar
@@ -82,8 +84,11 @@ if test $? -ne 0 ; then fail; fi
 # let the clock tick over
 sleep 1
 
-$bin/cook -nl -fp > /dev/null 2>&1
-if test $? -ne 0 ; then fail; fi
+$bin/cook -nl -fp > LOG 2>&1
+if test $? -ne 0 ; then cat LOG; fail; fi
+
+# the fingerprint file must exist at this point
+if test ! -r .cook.fp; then fail; fi
 
 # wait for clock to tick over
 sleep 2
@@ -91,8 +96,8 @@ sleep 2
 echo "hello" > bar
 if test $? -ne 0 ; then fail; fi
 
-$bin/cook -nl -fp > /dev/null 2>&1
-if test $? -ne 0 ; then fail; fi
+$bin/cook -nl -fp > LOG 2>&1
+if test $? -ne 0 ; then cat LOG; fail; fi
 
 #
 # Only definite negatives are possible.
