@@ -67,11 +67,15 @@ cat > Howto.cook << 'fubar'
 cc = [getenv CC]; if [not [cc]] then
 { cc = [find_command gcc]; if [not [cc]] then cc = cc; }
 
+exe = ;
+if [in [substr 1 6 [os]] CYGWIN NUTCRA] then
+	exe = .exe;
+
 set fingerprint update;
 search_list = . bl;
-test: test.c
+testprog[exe]: test.c
 {
-	[cc] -o test test.c;
+	[cc] -o [target] test.c;
 }
 fubar
 if test $? -ne 0 ; then fail; fi
@@ -84,7 +88,7 @@ if test $? -ne 0 ; then fail; fi
 $bin/cook -nl -silent
 if test $? -ne 0 ; then fail; fi
 
-./test
+./testprog
 if test $? -ne 42 ; then fail; fi
 
 #
@@ -93,7 +97,10 @@ if test $? -ne 42 ; then fail; fi
 mkdir bl
 if test $? -ne 0 ; then fail; fi
 
-mv test test.c bl
+mv test.c bl
+if test $? -ne 0 ; then fail; fi
+
+mv testprog.exe bl 2> LOG || mv testprog bl
 if test $? -ne 0 ; then fail; fi
 
 #
@@ -110,7 +117,7 @@ if test $? -ne 0 ; then fail; fi
 #
 sleep 1
 CC=${CC-gcc}
-$CC -o bl/test bl/test.c;
+$CC -o bl/testprog bl/test.c;
 if test $? -ne 0 ; then fail; fi
 
 #
@@ -119,7 +126,7 @@ if test $? -ne 0 ; then fail; fi
 $bin/cook -nl -silent
 if test $? -ne 0 ; then fail; fi
 
-./test
+./testprog
 if test $? -ne 0 ; then fail; fi
 
 #
